import asyncio
import os
import time
import smtplib
import mimetypes
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.base import MIMEBase
from email import encoders

from playwright.async_api import async_playwright

import pyautogui
from bs4 import BeautifulSoup
from rich.console import Console

from worker_automate_hub.api.client import get_config_by_name
from worker_automate_hub.utils.logger import logger


console = Console()


def send_email(smtp_server, smtp_port, smtp_user, smtp_password, message_text, subject, to):
    message = create_message_with_attachment(smtp_user, to, subject, message_text)
    send_message(smtp_server, smtp_port, smtp_user, smtp_password, smtp_user, to, message)


def create_message_with_attachment(sender, to, subject, message_text, file):
    message = MIMEMultipart()
    message['to'] = to
    message['from'] = sender
    message['subject'] = subject

    msg = MIMEText(message_text)
    message.attach(msg)

    if file:
        content_type, encoding = mimetypes.guess_type(file)

        if content_type is None or encoding is not None:
            content_type = 'application/octet-stream'

        main_type, sub_type = content_type.split('/', 1)
        with open(file, 'rb') as f:
            attachment = MIMEBase(main_type, sub_type)
            attachment.set_payload(f.read())
            encoders.encode_base64(attachment)
            attachment.add_header('Content-Disposition', 'attachment', filename=os.path.basename(file))
            attachment.add_header('Content-Transfer-Encoding', 'base64')
            message.attach(attachment)

    return message.as_string()


def send_message(smtp_server, smtp_port, smtp_user, smtp_password, sender, recipient, message):
    try:
        with smtplib.SMTP(smtp_server, smtp_port) as server:
            server.starttls()
            server.login(smtp_user, smtp_password)
            server.sendmail(sender, recipient, message)
            logger.info('Mensagem enviada com sucesso')
    except Exception as error:
        logger.error('Ocorreu um erro ao enviar a mensagem: %s' % error)


async def ecac_estadual_sc(task):
    """
    Processo que realiza a consulta de caixa postal do ECAC Estadual de SC.

    """
    try:
        # Obtém a resolução da tela
        screen_width, screen_height = pyautogui.size()
        console.print(f"Resolução da tela: Width: {screen_width} - Height{screen_height}...\n")
        console.print(f"Task:{task}")

        console.print("Realizando as validações inicias para execução do processo\n")
        console.print("Obtendo configuração para execução do processo ...\n")
        try:
            config = await get_config_by_name("Ecac_Estadual_SC")
            emails_to = config['conConfiguracao']['emails']
            user = config['conConfiguracao']['Usuario']
            password = config['conConfiguracao']['Senha']

            console.print("Obtendo configuração de email para execução do processo ...\n")
            smtp_config = await get_config_by_name("SMTP")

            smtp_server = smtp_config['conConfiguracao']['server']
            smtp_user = smtp_config['conConfiguracao']['user']
            smtp_pass = smtp_config['conConfiguracao']['password']
            smtp_port =  smtp_config['conConfiguracao']['port']

        except Exception as e:
            console.print(f'Erro ao obter as configurações para execução do processo, erro: {e}...\n')
            return {"sucesso": False, "retorno": f"Erro ao obter as configurações para execução do processo, erro: {e}"}
        

        async with async_playwright() as p:
            browser = await p.firefox.launch(headless=False)
            
            console.print("Iniciando o browser\n")
            context = await browser.new_context(accept_downloads=True)
            page = await context.new_page()

            await asyncio.sleep(3)

            console.print("Navegando até o site SAT... \n")
            await page.goto("https://sat.sef.sc.gov.br/tax.NET/sat.dtec.web/Acesso.aspx")
            console.print("Realizando login... \n")
            await page.get_by_placeholder("Usuário").fill(user)
            await page.get_by_placeholder("Senha").fill(password)
            await page.get_by_role("link", name="Entrar", exact=True).click()
            console.print("Sucesso ao realizar o login... \n")
            await page.get_by_role("link", name="person").click()
            console.print("Navegando até caixa de entrada... \n")
            await page.get_by_role("link", name=" Caixa de entrada").click()

            console.print("Extraindo mensagens... \n")
            await page.wait_for_selector('//*[@id="ext-gen47"]')
            time.sleep(3)

            div_content = await page.locator('//*[@id="ext-gen47"]').inner_html()

            console.print("Formatando saida de dados... \n")
            soup = BeautifulSoup(div_content, 'html.parser')

            rows = soup.select('tr')
            formatted_output = ''
            corpo_email = ''
            for row in rows:
                try:
                    id_mensagem = row.select_one('.x-grid3-col-0').text.strip()
                    status_lida = row.select_one('.x-grid3-col-1 img')['alt'].strip()
                    remetente = row.select_one('.x-grid3-col-2').text.strip()
                    tipo_mensagem = row.select_one('.x-grid3-col-3').text.strip()
                    data_envio = row.select_one('.x-grid3-col-4').text.strip()
                    assunto = row.select_one('.x-grid3-col-ASSUNTO').text.strip()

                    if status_lida == 'Não Lida':
                        formatted_output += (f"ID Mensagem: {id_mensagem}\n"
                                    f"Status: {status_lida}\n"
                                    f"Remetente: {remetente}\n"
                                    f"Tipo de Mensagem: {tipo_mensagem}\n"
                                    f"Data de Envio: {data_envio}\n"
                                    f"Assunto: {assunto}\n")
                except:
                    pass

            
            console.print("Enviando email para area de negócio \n")
            if formatted_output != '':
                corpo_email = f''' Caros(as), \nEspero que este e-mail o encontre bem! \n\n Abaixo, segue um resumo baseado na consulta à caixa postal referente ao estado de SC.
                \n Resultado da consulta: \n {formatted_output} \n '''
            else:
                corpo_email = f''' Caros(as), \nEspero que este e-mail o encontre bem! \n\n O processo foi executado para coleta de mensagens na caixa postal referente ao estado de SC, porém não foi encontrado nenhuma nova mensagem.'''

            try:
                await context.close()
                await browser.close()
            except Exception as e:
                console.print('Erro ao encerrar o browser... \n', e)

            try:
                send_email(smtp_server, smtp_port, smtp_user, smtp_pass, corpo_email, "Consulta ECAC_Estadual - SC", emails_to)
                log_msg = f"Processo concluído e e-mail disparado para área de negócio"
                console.print(log_msg)
                return {"sucesso": True, "retorno": log_msg}
            except Exception as e:
                log_msg = f"Processo concluído com sucesso, porém houve falha no envio do e-mail para área de negócio"
                console.print(log_msg)
                return {"sucesso": False, "retorno": log_msg}
            
    except Exception as e:
        log_msg = f"Erro Processo do Ecac Estadual SC: {e}"
        logger.error(log_msg)
        console.print(log_msg, style="bold red")
        return {"sucesso": False, "retorno": log_msg}


import asyncio
import warnings

import pyperclip
import pyautogui
from pywinauto.application import Application
from rich.console import Console

from worker_automate_hub.api.client import get_config_by_name
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.util import (
    find_target_position,
    import_nfe,
    kill_process,
    login_emsys,
    select_model_capa,
    take_screenshot,
    type_text_into_field,
    verify_nf_incuded,
    rateio_window,
    set_variable,
    importar_notas_outras_empresas,
    digitar_datas_emissao,
    digitar_numero_nota,
    worker_sleep,
)

pyautogui.PAUSE = 0.5
console = Console()



async def entrada_de_notas_39(task):
    """
    Processo que relazia entrada de notas no ERP EMSys(Linx).

    """
    try:
        #Get config from BOF
        config = await get_config_by_name("login_emsys")
        console.print(task)
       
        #Seta config entrada na var nota para melhor entendimento
        nota = task['configEntrada']
        multiplicador_timeout = int(float(task["sistemas"][0]["timeout"]))
        set_variable("timeout_multiplicador", multiplicador_timeout)

        #Abre um novo emsys
        await kill_process("EMSys")
        app = Application(backend='win32').start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        warnings.filterwarnings("ignore", category=UserWarning, message="32-bit application should be automated using 32-bit Python")
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(config['conConfiguracao'], app, task)

        if return_login['sucesso'] == True:
            type_text_into_field('Nota Fiscal de Entrada', app['TFrmMenuPrincipal']['Edit'], True, '50')
            pyautogui.press('enter')
            await worker_sleep(1)
            pyautogui.press('enter')
            console.print(f"\nPesquisa: 'Nota Fiscal de Entrada' realizada com sucesso", style="bold green")
        else:
            logger.info(f"\nError Message: {return_login["retorno"]}")
            console.print(f"\nError Message: {return_login["retorno"]}", style="bold red")
            return return_login
        
        await worker_sleep(10)
        #Procura campo documento
        model  = select_model_capa()
        if model['sucesso'] == True:
            console.log(model['retorno'], style='bold green')
        else:
            return {"sucesso": False, "retorno": f"{model['retorno']}"}

        #Clica em 'Importar-Nfe'
        imported_nfe  = await import_nfe()
        if imported_nfe['sucesso'] == True:
            console.log(imported_nfe['retorno'], style='bold green')
        else:
            return {"sucesso": False, "retorno": f"{import_nfe['retorno']}"}

        # Clica em Notas de Outras Empresas
        pyautogui.click(824, 547)
        await worker_sleep(2)

        #Clica em  'OK' para selecionar
        pyautogui.click(975, 674)
        await worker_sleep(2)

        empresa = str(int(nota['cnpjFornecedor'][8:12]))
        await importar_notas_outras_empresas(nota["dataEmissao"], nota["numeroNota"], empresa)

        # #Digita datas de emissão
        # data_emissao = nota['dataEmissao'].replace('/', '')
        # digitar_datas_emissao(data_emissao)

        #Digita numero da nota
        # numero_nota = nota['numeroNota']
        # digitar_numero_nota(numero_nota)
        #await worker_sleep(100)
  
        # #clica na nota
        # pyautogui.click(791,483)
        # await worker_sleep(2)

        # #Clica em 'Importar'
        # screenshot_path = take_screenshot()
        # field = find_target_position(screenshot_path, "cancelar",0 ,100 , 15)
        # if field == None:
        #     return {"sucesso": False, "retorno": f"Não foi possivel encontrar o botão importar"}
        # pyautogui.click(field)
        
        await worker_sleep(10)

        # Identifica se ja foi importada
        screenshot_path = take_screenshot()
        field = find_target_position(screenshot_path, "Mensagem", 0, 0, 10)
        if field == None:
            ...
        else:
            console.log("Nota já lançada ou não encontrada.", style='bold green')
            return {"sucesso": False, "retorno": f"Nota já lançada ou não encontrada."}

        #Tela "Infromações para importação da Nota Fiscal Eletrônica"
        screenshot_path = take_screenshot()

        if nota['existeDespesa'] == 'Sim':
            #Digita natureza da operação
            pyautogui.hotkey('tab')
            pyautogui.write("1152")
            pyautogui.hotkey('tab')

            #Digita tipo despesa
            pyautogui.click(800, 479)
            pyautogui.click(field)
            pyautogui.write("291")
            pyautogui.hotkey("tab")

            #Marca "Manter Natureza de operação selecionada"
            pyautogui.click(963, 578)
            #Marca "Manter Calculo PIS/COFINS"
            pyautogui.click(705, 667)

        #Clica em OK

        pyautogui.click(704, 665)
        await worker_sleep(5)
        pyautogui.click(1100, 730)
        await worker_sleep(20)

        try:
            # Tratativa de "itens com ncm divergente" - PASSO 27.1 DA IT
            screenshot_path = take_screenshot()
            console.log("Procurando itens com ncm divergente", style='bold yellow')
            itens = find_target_position(screenshot_path, 'NCM', 0, 0, 15)

            if itens is not None:
                pyautogui.click(1000, 568)
                console.log("Clicou em não nos itens com ncm divergente", style='bold yellow')
            else:
                console.log("0 Itens com ncm divergente", style='bold green')

            await worker_sleep(10)

            # Itens não localizados - Clica em Sim
            screenshot_path = take_screenshot()
            console.log("Verificando tela de Itens Não Localizados", style='bold yellow')
            itens_nloc = find_target_position(screenshot_path, 'associar', 0, 0, 15)

            if itens_nloc:
                pyautogui.click(920, 562)
                await worker_sleep(3)
                pyautogui.click(920, 304)
                pyautogui.hotkey("ctrl", "c")
                clipboard = pyperclip.paste()

                if clipboard:
                    console.log("Itens não localizados na nota.", style='bold red')

            # Tratativa "Selecionar itens fornecedor" - PASSO 27.3 da it
            multi_ref = True
            while multi_ref:
                screenshot_path = take_screenshot()
                field = find_target_position(screenshot_path, "fornecedor", 0, 0, 5)

                if field:
                    pyautogui.click(1075, 624)
                    await worker_sleep(20)
                else:
                    multi_ref = False

        except Exception as e:
            console.print(f"Erro durante a execução: {e}", style="red")
            await worker_sleep(3)

        #Seleciona pagamento
        console.log("Seleciona Pagamento", style='bold yellow')
        pyautogui.click(623, 374)
        await worker_sleep(1)
        pyautogui.click(889, 349)
        await worker_sleep(1)
        pyautogui.write("27")
        await worker_sleep(1)
        pyautogui.hotkey("enter")

        #Digita "Valor"

        pyautogui.click(1285, 352)
        await worker_sleep(1)
        pyautogui.hotkey("ctrl", "a")
        pyautogui.hotkey("del")
        await worker_sleep(1)
        pyautogui.write(nota['valorNota'])
        pyautogui.click(593, 302)
        await worker_sleep(10)

        # Nota com itens bloqueados?
        pyautogui.click(1007, 562)
        await worker_sleep(1)
        pyautogui.click(1358, 751)
        await worker_sleep(1)

        pyautogui.click(1111, 608)
        await worker_sleep(1)
        pyautogui.click(1234, 628)
        await worker_sleep(60)

        #Clica para lançar a nota
        pyautogui.click(971, 569)
        await worker_sleep(1)
        console.log("Clicou pra lançar nota", style='bold green')

        #Variação Maxima de custo
        pyautogui.click(1234, 633)
        console.log("Clicou 'OK' variacao maxima de custo", style='bold green')

        #Quantidade de tempo alta devido o emsys ser lento
        console.log("Aguardando delay...", style='bold green')
        await worker_sleep(30)

        console.log("Verificando se nota foi lançada", style='bold green')
        retorno = verify_nf_incuded()
        if retorno:
            pyautogui.click(959, 564)
            return {"sucesso": True, "retorno": f"Nota Lançada com sucesso!"}
        else:
            console.log("Verficando se necessita realizar o rateio")
        

        if nota['existeDespesa'] == 'Sim':
            console.log("Inicializando o rateio", style='bold green')
            await rateio_window(nota)
            asyncio.sleep(10)

        #Aviso CFOP
        console.log("Verificando aviso CFOP", style='bold green')
        screenshot_path = take_screenshot()
        field = None
        field = find_target_position(screenshot_path, "mesmo", 0, 0, 8)
        if field == None:
            console.print("Nota com CFOP correto")
        else:
            #Clica em OK
            pyautogui.click(1096, 603)
            #Clica em "Principal"
            field = find_target_position(screenshot_path, "Principal", 0, 0, 8)
            pyautogui.click(field)
            #Selecio no "Principal" a NOP correta
            field = find_target_position(screenshot_path, "Inscrição", 20, 0, 8)
            pyautogui.click(field)
            pyautogui.write("1152")
            #Clica para lançar a nota
            pyautogui.click(594, 299)

            #Clica novamente em ok na variação de custo      
            field = find_target_position(screenshot_path, "ultrapassam", 0, 0, 8)
            if field == None:
                console.print("Nota sem itens com variação de custo")
            else:
                pyautogui.click(1234, 633)

        #Clica para lançar a nota
        pyautogui.click(594, 299)
        await worker_sleep(5)

        #Verifica se a info 'Nota fiscal incluida' está na tela   
        retorno = verify_nf_incuded()
        if retorno:
            pyautogui.click(959, 564)
            return {"sucesso": True, "retorno": f"Nota Lançada com sucesso!"}
        else:
            return {"sucesso": False, "retorno": f"Erro ao lançar nota"}

    except Exception as ex:
        observacao = f"Erro Processo Entrada de Notas: {str(ex)}"
        logger.error(observacao)
        console.print(observacao, style="bold red")
        return {"sucesso": False, "retorno": observacao}



import asyncio
import requests
from pathlib import Path
import io
import shutil
import uuid
import os
import time
from datetime import datetime

import zipfile
import smtplib
import mimetypes
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.base import MIMEBase
from email import encoders

from playwright.async_api import async_playwright

import pyautogui
from rich.console import Console

from worker_automate_hub.api.client import get_config_by_name
from worker_automate_hub.api.client import sync_get_config_by_name
from worker_automate_hub.utils.logger import logger

from worker_automate_hub.utils.get_creds_gworkspace import GetCredsGworkspace
from googleapiclient.http import MediaIoBaseDownload
from googleapiclient.discovery import build


console = Console()


def find_and_click(image_path, description, confidence=0.8, write_text=None, wait_time=2):
    element = pyautogui.locateCenterOnScreen(image_path, confidence=confidence)
    if element:
        pyautogui.click(element)
        console.print(f"[green]'{description}' encontrado e clicado.[/green]")
        if write_text:
            pyautogui.write(write_text, interval=0.05)
            console.print(f"[green]Texto '{write_text}' inserido em '{description}'.[/green]")
        time.sleep(wait_time)
    else:
        raise Exception(f"[red]Elemento '{description}' não encontrado![/red]")
    

def compactar_diretorio(diretorio, nome_arquivo_zip):
    arquivos = [os.path.join(diretorio, arquivo) for arquivo in os.listdir(diretorio) if os.path.isfile(os.path.join(diretorio, arquivo))]

    with zipfile.ZipFile(nome_arquivo_zip, 'w') as zipf:
        for arquivo in arquivos:
            zipf.write(arquivo, os.path.basename(arquivo))


def send_email(smtp_server, smtp_port, smtp_user, smtp_password, message_text, subject, to, diretorio):

    nome_arquivo_zip = 'Consulta ECAC Estadual SP.zip'
    full_path = os.path.join(diretorio, nome_arquivo_zip)
    compactar_diretorio(diretorio, full_path)

    message = create_message_with_attachment(smtp_user, to, subject, message_text, full_path)
    send_message(smtp_server, smtp_port, smtp_user, smtp_password, smtp_user, to, message)


def create_message_with_attachment(sender, to, subject, message_text, file):
    message = MIMEMultipart()
    message['to'] = to
    message['from'] = sender
    message['subject'] = subject

    msg = MIMEText(message_text)
    message.attach(msg)

    if file:
        content_type, encoding = mimetypes.guess_type(file)

        if content_type is None or encoding is not None:
            content_type = 'application/octet-stream'

        main_type, sub_type = content_type.split('/', 1)
        with open(file, 'rb') as f:
            attachment = MIMEBase(main_type, sub_type)
            attachment.set_payload(f.read())
            encoders.encode_base64(attachment)
            attachment.add_header('Content-Disposition', 'attachment', filename=os.path.basename(file))
            attachment.add_header('Content-Transfer-Encoding', 'base64')
            message.attach(attachment)

    # raw_message = base64.urlsafe_b64encode(message.as_bytes())
    # raw_message = raw_message.decode()
    # return {'raw': raw_message}
    return message.as_string()


def send_message(smtp_server, smtp_port, smtp_user, smtp_password, sender, recipient, message):
    try:
        with smtplib.SMTP(smtp_server, smtp_port) as server:
            server.starttls()
            server.login(smtp_user, smtp_password)
            server.sendmail(sender, recipient, message)
            logger.info('Mensagem enviada com sucesso')
    except Exception as error:
        logger.error('Ocorreu um erro ao enviar a mensagem: %s' % error)


async def ecac_estadual_sp(task):
    """
    Processo que realiza a consulta de caixa postal do ECAC Estadual de São Paulo para os certificado SIM DISTRIBUIDORA e SIM LUBRIFICANTES.

    """
    try:
        # Obtém a resolução da tela
        screen_width, screen_height = pyautogui.size()
        console.print(f"Resolução da tela: Width: {screen_width} - Height{screen_height}...\n")
        console.print(f"Task:{task}")

        console.print("Realizando as validações inicias para execução do processo\n")
        console.print("Criando o diretório temporario ...\n")

        cd = Path.cwd()
        temp_dir = f"{cd}\\temp_certificates\\{str(uuid.uuid4())}"
        if not os.path.exists(temp_dir):
            os.makedirs(temp_dir)
        
        pdf_dir = os.path.join(temp_dir, "Arquivos_PDFs")
        if not os.path.exists(pdf_dir):
            os.makedirs(pdf_dir)

        console.print("Obtendo configuração para execução do processo ...\n")
        try:
            config = await get_config_by_name("Ecac_Estadual_SP")
            emails_to = config['conConfiguracao']['emails']
            certificado_path = config['conConfiguracao']['CertificadoPath']
            certificado_files = config['conConfiguracao']['CertificadoFile']

            console.print("Obtendo configuração de email para execução do processo ...\n")
            smtp_config = await get_config_by_name("SMTP")

            smtp_server = smtp_config['conConfiguracao']['server']
            smtp_user = smtp_config['conConfiguracao']['user']
            smtp_pass = smtp_config['conConfiguracao']['password']
            smtp_port =  smtp_config['conConfiguracao']['port']

            get_gcp_token = sync_get_config_by_name("GCP_SERVICE_ACCOUNT")
            get_gcp_credentials = sync_get_config_by_name("GCP_CREDENTIALS")
        except Exception as e:
            console.print(f'Erro ao obter as configurações para execução do processo, erro: {e}...\n')
            return {"sucesso": False, "retorno": f"Erro ao obter as configurações para execução do processo, erro: {e}"}


        gcp_credencial = GetCredsGworkspace(token_dict=get_gcp_token['conConfiguracao'], credentials_dict=get_gcp_credentials['conConfiguracao'])
        creds = gcp_credencial.get_creds_gworkspace()

        if not creds:
            console.print(f'Erro ao obter autenticação para o GCP...\n')
            return {"sucesso": False, "retorno": f"Erro ao obter autenticação para o GCP"}


        console.print("Verificando a existência do diretório com locators...\n")
        if os.path.exists("assets/ecac_federal"):
            console.print('Diretório existe..\n')
        else:
            console.print('Diretório não existe..\n')
            return {"sucesso": False, "retorno": f"Não foi possivel encontrar o diretório com os locators para continuidade do processo, diretório: 'assets/ecac_federal'"}


        console.print('Verificando a existência dos arquivos de certificados no Google Drive...\n')
        drive_service = build("drive", "v3", credentials=creds)
        query = f"'{certificado_path}' in parents"
        results = drive_service.files().list(
            q=query,
            pageSize=1000,
            supportsAllDrives=True,  # Habilita suporte para Shared Drives
            includeItemsFromAllDrives=True,  # Inclui itens de todos os drives
            fields="files(id, name)"
        ).execute()

        items = results.get('files', [])

        if not items:
            console.print(f'Nenhum certificado encontrado...\n')
            return {"sucesso": False, "retorno": f"Nenhum certificado encontrado no diretório do Google Drive"}

        corpo_email = '''
        <html>
        <body>
            <p>Caros(as),</p>
            <p>Espero que este e-mail o encontre bem!</p>
            <p>Abaixo, segue um resumo baseado na consulta à caixa postal referente ao estado de SP.</p>
            <p>Resultado da consulta:</p>
        '''
        error_count = 0
        
        for row in certificado_files:
            try:
                empresa = row['Empresa']
                certificado = row['Certificado']
                #validade = row['Validade']
                senha = row['Senha']

                console.print(f'Procurando certificado: {certificado} para a empresa {empresa}...\n')
                drive_service = build("drive", "v3", credentials=creds)
                query = f"'{certificado_path}' in parents and name='{certificado}'"
                results = drive_service.files().list(
                    q=query,
                    pageSize=1000,
                    supportsAllDrives=True,  # Habilita suporte para Shared Drives
                    includeItemsFromAllDrives=True,  # Inclui itens de todos os drives
                    fields="files(id, name)"
                ).execute()

                items = results.get('files', [])

                if items:
                    console.print(f'Certificado: {certificado} para a empresa {empresa} encontrado...\n')
                    file_id = items[0]['id']
                    console.print(f'Certificado {certificado} encontrado. Iniciando download...\n')

                    file_path = os.path.join(temp_dir, certificado)

                    request = drive_service.files().get_media(fileId=file_id)

                    fh = io.FileIO(file_path, 'wb')
                    downloader = MediaIoBaseDownload(fh, request)
    
                    done = False
                    while done is False:
                        status, done = downloader.next_chunk()
                        console.print(f"Download {int(status.progress() * 100)}%.")
                    
                    fh.close()

                    console.print(f"Certificado {certificado} baixado com sucesso.\n")

                    async with async_playwright() as p:
                        browser = await p.firefox.launch(headless=False)
                        console.print("Iniciando o browser")
                        context = await browser.new_context(accept_downloads=True) 
                        page = await context.new_page() 

                        await asyncio.sleep(3)

                        await page.goto('about:preferences#privacy')

                        locator_path = "assets/ecac_federal"

                        element_find_settings_image = f'{locator_path}/FindInSettings.PNG' 
                        element_view_certificates = f'{locator_path}/ViewCertificates.PNG'
                        element_your_certificates = f'{locator_path}/YourCertificates.PNG'
                        element_import_certificates = f'{locator_path}/ImportCertificate.PNG'
                        element_insert_path_certificates = f'{locator_path}/InsertPathCertificate.PNG'
                        element_open_certificates = f'{locator_path}/OpenCertificate.PNG'
                        element_pwd_certificates = f'{locator_path}/EnterPasswordCertificate.PNG'
                        element_sign_in_certificates = f'{locator_path}/SignCertificate.PNG'
                        element_confirm_certificates = f'{locator_path}/Confirm.PNG'
                        element_sign_in_certificates = f'{locator_path}/SignCertificate.PNG'
                        element_popup_certificate = f'{locator_path}/ImportCertificate_popup.PNG'


                        console.print("Importando o certificado no browser (playwright) ")
                        # Configurações
                        find_and_click(element_find_settings_image, "Configurações")
                        pyautogui.write('Cert', interval=0.05)
                        await asyncio.sleep(1)

                        # Certificados
                        find_and_click(element_view_certificates, "Certificados")
                        await asyncio.sleep(1)

                        # Meus Certificados
                        find_and_click(element_your_certificates, "Meus Certificados")
                        await asyncio.sleep(1)

                        # Importar Certificado
                        find_and_click(element_import_certificates, "Importar Certificado")
                        await asyncio.sleep(2)

                        # Inserir Caminho do Certificado e escrever o caminho do arquivo
                        find_and_click(element_insert_path_certificates, "Inserir Caminho do Certificado", write_text=file_path)
                        await asyncio.sleep(2)

                        # Abrir Certificado
                        find_and_click(element_open_certificates, "Abrir Certificado")
                        await asyncio.sleep(2)

                        # Inserir Senha do Certificado
                        find_and_click(element_pwd_certificates, "Inserir Senha do Certificado", write_text=senha)
                        await asyncio.sleep(2)

                        # Assinar Certificado
                        find_and_click(element_sign_in_certificates, "Assinar Certificado")
                        await asyncio.sleep(2)

                        # Confirmar Importação
                        find_and_click(element_confirm_certificates, "Confirmar Importação")
                        await asyncio.sleep(2)

                        element_popup_certificate = pyautogui.locateCenterOnScreen(element_popup_certificate, confidence=0.8)

                        if not element_popup_certificate:
                            await asyncio.sleep(3)
                            
                            try:
                                await page.goto('https://www.dec.fazenda.sp.gov.br/DEC/UCLogin/login.aspx', timeout=1000)
                                await asyncio.sleep(3)
                                await page.keyboard.press("Enter")
                            except:
                                await page.keyboard.press("Enter")
                                await page.goto('https://www.dec.fazenda.sp.gov.br/DEC/UCLogin/login.aspx', timeout=1000)
                                await asyncio.sleep(3)
                                await page.keyboard.press("Enter")
                            
                            await page.goto("https://www.dec.fazenda.sp.gov.br/DEC/UCServicosDisponiveis/ExibeServicosDisponiveis.aspx")
                            #await page.locator("#ConteudoPagina_tabContainerServicos_tabResponsavel_gvServicosSocio_cmdCPE_0").click()
                            await page.locator('//*[@id="ConteudoPagina_HyperLink5"]').click()
                            await page.get_by_role("link", name="Ativos").click()
                            await page.get_by_role("button", name="Todos").click()

                            message_table = await page.locator('//*[@id="ConteudoPagina_tabContainerCaixaPostal_tabAtivas_ucItensAtivos_tabContainerCaixaPostal_tabMensagens_gvMensagens"]')
                            rows = await message_table.locator("tbody tr")
                            row_count = await rows.count()

                            data_atual = datetime.now().strftime("%d/%m/%Y")

                            for i in range(row_count):
                                input_element = rows.nth(i).locator('td:nth-child(2) input[type="hidden"]')
                                hyperlink_value = await input_element.get_attribute("value")
                                
                                data_envio_element = rows.nth(i).locator('td:nth-child(5)')
                                data_envio_text = await data_envio_element.inner_text()

                                data_envio_formatada = data_envio_text.split(" ")[0]

                                if data_envio_formatada == data_atual:

                                    default_url = f"https://www.dec.fazenda.sp.gov.br/DEC/Comum/RelatorioMensagem.aspx?args={hyperlink_value}"

                                    response = requests.get(default_url)

                                    if response.status_code == 200:
                                        pdf_filename = os.path.join(pdf_dir, f"Arquivo_CaixaPostal_{i}.pdf")
                                        with open(pdf_filename, 'wb') as f:
                                            f.write(response.content)
                                    
                                        corpo_email += f'''
                                                <p>O arquivo foi baixado com sucesso e está salvo como {pdf_filename} para a empresa {empresa}.</p>
                                            '''
                                    else:
                                        corpo_email += f'''
                                        <p>Havia um arquivo disponível para a empresa {empresa}, mas não foi possível baixá-lo. Código de status: {response.status_code}.</p>
                                    '''
                                else:
                                    corpo_email += f'''
                                        <p>Mensagem disponivel para {empresa}, porém ja passou o limite da data para acesso.</p>
                                    '''

                            await context.close()
                            await browser.close()

                else:
                    console.print(f'Certificado não encontrado no Driver {certificado}...\n')
                    corpo_email += f"""
                        <p>Certificado para a empresa {empresa} não encontrado no Google Drive.</p>
                        """
                    error_count +=1
            except Exception as e:
                console.print(f'Erro na consulta para o certificado {certificado}, empresa {empresa}, erro{e}...\n')
                error_count +=1
        if error_count < len(certificado_files):
            try:
                corpo_email += '''
                    </body>
                    </html>
                    '''
                
                send_email(smtp_server, smtp_port, smtp_user, smtp_pass, corpo_email, "Consulta ECAC_Estadual - SP", emails_to,pdf_dir)
                log_msg = f"Processo concluído e e-mail disparado para área de negócio"
                console.print(log_msg)
                return {"sucesso": True, "retorno": log_msg}
            except Exception as e:
                log_msg = f"Processo concluído com sucesso, porém houve falha no envio do e-mail para área de negócio"
                console.print(log_msg)
                return {"sucesso": False, "retorno": log_msg}
        else:
            log_msg = f"Não foi possivel executar o processo para o Ecac Estadual SP"
            console.print(log_msg)
            return {"sucesso": False, "retorno": log_msg}

    except:
        if os.path.exists(temp_dir):
            shutil.rmtree(temp_dir)
        
        if not os.path.exists(pdf_dir):
            shutil.rmtree(pdf_dir)

        log_msg = f"Erro Processo do Ecac Estadual SP: {e}"
        logger.error(log_msg)
        console.print(log_msg, style="bold red")
        return {"sucesso": False, "retorno": log_msg}


import warnings

import pyautogui
from pywinauto.application import Application
from pywinauto_recorder.player import (
    set_combobox,
)
from rich.console import Console
from worker_automate_hub.api.client import sync_get_config_by_name

from worker_automate_hub.api.client import get_config_by_name
from worker_automate_hub.config.settings import load_env_config
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.util import (
    import_nfe,
    kill_process,
    login_emsys,
    type_text_into_field,
    incluir_registro,
    set_variable,
    get_xml,
    delete_xml,
    download_xml,
    worker_sleep,
    verify_nf_incuded,
)

pyautogui.PAUSE = 0.5
console = Console()


async def entrada_de_notas_207(task):
    """
    Processo que relazia entrada de notas no ERP EMSys(Linx).

    """
    try:
        #Get config from BOF
        config = await get_config_by_name("login_emsys")
        console.print(task)
       
        #Seta config entrada na var nota para melhor entendimento
        nota = task['configEntrada']
        multiplicador_timeout = int(float(task["sistemas"][0]["timeout"]))
        set_variable("timeout_multiplicador", multiplicador_timeout)

        #Abre um novo emsys
        await kill_process("EMSys")
        app = Application(backend='win32').start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        warnings.filterwarnings("ignore", category=UserWarning, message="32-bit application should be automated using 32-bit Python")
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(config['conConfiguracao'], app, task)

        if return_login['sucesso'] == True:
            type_text_into_field('Nota Fiscal de Entrada', app['TFrmMenuPrincipal']['Edit'], True, '50')
            pyautogui.press('enter')
            await worker_sleep(2)
            pyautogui.press('enter')
            console.print(f"\nPesquisa: 'Nota Fiscal de Entrada' realizada com sucesso", style="bold green")
        else:
            logger.info(f"\nError Message: {return_login["retorno"]}")
            console.print(f"\nError Message: {return_login["retorno"]}", style="bold red")
            return return_login
        
        await worker_sleep(6)
        #Procura campo documento
        console.print('Navegando pela Janela de Nota Fiscal de Entrada...\n')
        app = Application().connect(title="Nota Fiscal de Entrada")
        main_window = app["Nota Fiscal de Entrada"]

        console.print("Controles encontrados na janela 'Nota Fiscal de Entrada, navegando entre eles...\n")
        panel_TNotebook = main_window.child_window(class_name="TNotebook", found_index=0)
        panel_TPage = panel_TNotebook.child_window(class_name="TPage", found_index=0)
        panel_TPageControl = panel_TPage.child_window(class_name="TPageControl", found_index=0)
        panel_TTabSheet = panel_TPageControl.child_window(class_name="TTabSheet", found_index=0)
        combo_box_tipo_documento = panel_TTabSheet.child_window(class_name="TDBIComboBox", found_index=1)
        combo_box_tipo_documento.click()
        console.print("Clique select box, Tipo de documento realizado com sucesso, selecionando o tipo de documento...\n")

        await worker_sleep(2)

        set_combobox("||List", "NOTA FISCAL DE ENTRADA ELETRONICA - DANFE")
        console.print("Tipo de documento 'NOTA FISCAL DE ENTRADA ELETRONICA - DANFE', selecionado com sucesso...\n")

        await worker_sleep(4)

        #Clica em 'Importar-Nfe'
        imported_nfe  = await import_nfe()
        if imported_nfe['sucesso'] == True:
            console.log(imported_nfe['retorno'], style='bold green')
        else:
            return {"sucesso": False, "retorno": f"{imported_nfe['retorno']}"}

        await worker_sleep(10)

        # Download XML
        get_gcp_token = sync_get_config_by_name("GCP_SERVICE_ACCOUNT")
        get_gcp_credentials = sync_get_config_by_name("GCP_CREDENTIALS")
        env_config, _ = load_env_config()

        download_result = await download_xml(env_config["XML_DEFAULT_FOLDER"], get_gcp_token, get_gcp_credentials, nota["nfe"])
        if download_result['sucesso'] == True:
            console.log('Download do XML realizado com sucesso', style='bold green')
        else:
            return {"sucesso": False, "retorno": f"{download_result['retorno']}"}

        await get_xml(nota["nfe"])
        await worker_sleep(3)

        # Deleta o xml
        await delete_xml(nota["nfe"])
        
        #VERIFICANDO A EXISTENCIA DE WARNINGS
        await worker_sleep(4)
        try:
            console.print("Verificando a existencia de warning após a importação do xml...\n")
            app = Application().connect(title="Warning")
            main_window = app["Warning"]
            
            console.print("Clicando em NO, para andamento do processo...\n")
            btn_no = main_window.child_window(title="&No")
            if btn_no.exists() and btn_no.is_enabled():
                btn_no.click()
                await worker_sleep(7)
            else:
                console.print("Warning - Erro após a importação do arquivo...\n")
                return {"sucesso": False, "retorno": 'Warning - Erro após a importação do arquivo, não foi encontrado o botão No para andamento do processo... \n'}
                
        except:
            console.print("Não possui nenhum warning após a importação do xml...\n")
        
        #VERIFICANDO A EXISTENCIA DE ERRO
        try:
            app = Application().connect(title="Erro")
            main_window = app["Erro"]
            await worker_sleep(7)
            all_controls_from_error = main_window.children()
            capturar_proxima_mensagem = False

            for control in all_controls_from_error:
                control_text = control.window_text()

                if "Mensagem do Banco de Dados" in control_text:
                    capturar_proxima_mensagem = (
                        True
                    )

                elif capturar_proxima_mensagem:
                    return {"sucesso": False, "retorno": f'Pop-UP Erro após a importação do arquivo, erro do banco de dados: {control_text} \n'}

                if "XML já foi importado anteriormente" in control_text:
                    return {"sucesso": False, "retorno": f'Pop-UP Erro após a importação do arquivo, Nota descartada {control_text}... \n'}           
            
        except:
            console.print("Não possui nenhuma mensagem de erro após a importação do xml...\n")

        app = Application().connect(title="Informações para importação da Nota Fiscal Eletrônica")
        main_window = app["Informações para importação da Nota Fiscal Eletrônica"]

        #INTERAGINDO COM A NATUREZA DA OPERACAO
        cfop = int(nota["cfop"])
        console.print(f"Inserindo a informação da CFOP, caso se aplique {cfop} ...\n")
        if cfop == 5655 or str(cfop).startswith("56"):
            combo_box_natureza_operacao = main_window.child_window(class_name="TDBIComboBox", found_index=0)
            combo_box_natureza_operacao.click()

            await worker_sleep(3)
            set_combobox("||List", "1652-COMPRA DE MERCADORIAS- 1.652")
            await worker_sleep(3)
        
        else:
            console.print("Erro mapeado, CFOP diferente de 5655 ou 56, necessario ação manual ou ajuste no robo...\n")
            return {"sucesso": False, "retorno": f"Erro mapeado, CFOP diferente de 5655 ou 56, necessario ação manual ou ajuste no robo"}


        #INTERAGINDO COM O CAMPO ALMOXARIFADO
        filialEmpresaOrigem = nota["filialEmpresaOrigem"]
        console.print(f"Inserindo a informação do Almoxarifado {filialEmpresaOrigem} ...\n")
        option_t_edit_class = main_window.children(class_name="TDBIEditCode")
        enabled_t_edit_class = [
            almoxarifado for almoxarifado in option_t_edit_class if almoxarifado.is_enabled()
        ]

        if (
            len(enabled_t_edit_class) > 1
        ):
            almoxarifado = enabled_t_edit_class[1]
            valor_almoxarifado = filialEmpresaOrigem + "50"
            console.print(f"Interagindo com o campo Almoxarifado, inserindo: {valor_almoxarifado}")
            almoxarifado.set_edit_text(valor_almoxarifado)
            await worker_sleep(1)
            almoxarifado.type_keys('{TAB}')
        else:
            console.print("Não há campos suficientes para interagir com Almoxarifado...\n")


        await worker_sleep(3)
        #INTERAGINDO COM CHECKBOX Utilizar unidade de agrupamento dos itens
        console.print("Verificando se a nota é do fornecedor SIM Lubrificantes \n")
        fornecedor = nota["nomeFornecedor"]
        console.print(f"Fornecedor: {fornecedor} ...\n")
        if "sim lubrificantes" in fornecedor.lower():
            console.print(f"Sim, nota emitida para: {fornecedor}, marcando o agrupar por unidade de medida...\n")
            checkbox = main_window.child_window(
                title="Utilizar unidade de agrupamento dos itens",
                class_name="TCheckBox",
            )
            if not checkbox.is_checked():
                checkbox.check()
                console.print("Realizado o agrupamento por unidade de medida... \n")
        else:
            console.print("Não foi necessario realizar o agrupamento por unidade de medida... \n")

        await worker_sleep(2)
        console.print("Clicando em OK... \n")
        btn_ok = main_window.child_window(title="Ok")
        btn_ok.click()
        await worker_sleep(6)

        # Inclui registro
        console.print(f"Incluindo registro...\n")
        try:
            ASSETS_PATH = "assets"
            inserir_registro = pyautogui.locateOnScreen(ASSETS_PATH + "\\entrada_notas\\IncluirRegistro.png", confidence=0.8)
            pyautogui.click(inserir_registro)
        except Exception as e:
            console.print(f"Não foi possivel incluir o registro utilizando reconhecimento de imagem, Error: {e}...\n tentando inserir via posição...\n")
            incluir_registro()

        #Verifica se a info 'Nota fiscal incluida' está na tela   
        retorno = await verify_nf_incuded()
        if retorno:
            console.print("\nNota lançada com sucesso...", style="bold green")
            return {"sucesso": True, "retorno": f"Nota Lançada com sucesso!"}
        else:
            console.print("Erro ao lançar nota", style="bold red")
            return {"sucesso": False, "retorno": f"Erro ao lançar nota"}

    except Exception as ex:
        observacao = f"Erro Processo Entrada de Notas: {str(ex)}"
        logger.error(observacao)
        console.print(observacao, style="bold red")
        return {"sucesso": False, "retorno": observacao}

    finally:
        await kill_process("EMSys")



import asyncio
import math
import os
import io
import re
import subprocess
import time
import warnings
import getpass
from pathlib import Path

import aiohttp
import cv2
import psutil
import pyautogui
import pyperclip
from PIL import Image 
from prompt_toolkit.shortcuts import checkboxlist_dialog
from pytesseract import pytesseract
from rich.console import Console

from worker_automate_hub.config.settings import load_worker_config
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.updater import get_installed_version

from worker_automate_hub.utils.get_creds_gworkspace import GetCredsGworkspace
from googleapiclient.discovery import build
from googleapiclient.http import MediaIoBaseDownload

console = Console()

ASSETS_PATH = "assets"

global_variables = {}

def set_variable(key, value):
    global_variables[key] = value

def get_variable(key):
    return global_variables.get(key, None)

async def worker_sleep(multiplier):
    # Função que espera o tempo configurado multiplicado por um fator
    timeout_multiplicador = get_variable("timeout_multiplicador")
    if timeout_multiplicador is None:
        raise ValueError("O timeout multiplicador não foi definido")
    console.log(f"Aguardando {(timeout_multiplicador * multiplier)} segundos...", style="bold yellow")
    await asyncio.sleep(timeout_multiplicador * multiplier)


async def get_system_info():
    worker_config = load_worker_config()
    max_cpu = psutil.cpu_percent(interval=10.0)
    cpu_percent = psutil.cpu_percent(interval=1.0)
    memory_info = psutil.virtual_memory()

    return {
        "uuidRobo": worker_config["UUID_ROBO"],
        "maxCpu": f"{max_cpu}",
        "maxMem": f"{memory_info.total / (1024 ** 3):.2f}",
        "usoCpu": f"{cpu_percent}",
        "usoMem": f"{memory_info.used / (1024 ** 3):.2f}",
        "situacao": "{'status': 'em desenvolvimento'}",
    }


async def get_new_task_info():
    worker_config = load_worker_config()
    atual_version = get_installed_version("worker-automate-hub")
    return {
        "uuidRobo": worker_config["UUID_ROBO"],
        "versao": atual_version,
    }


def multiselect_prompt(options, title="Select options"):
    result = checkboxlist_dialog(
        values=[(option, option) for option in options],
        title=title,
        text="Use space to select multiple options.\nPress Enter to confirm your selection.",
    ).run()

    if result is None:
        console.print("[red]No options selected.[/red]")

async def kill_process(process_name: str):
    try:
        # Obtenha o nome do usuário atual
        current_user = os.getlogin()

        # Liste todos os processos do sistema
        result = await asyncio.create_subprocess_shell(
            f'tasklist /FI "USERNAME eq {current_user}" /FO CSV /NH',
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )

        stdout, stderr = await result.communicate()

        if result.returncode != 0:
            err_msg = f"Erro ao listar processos: {stderr.decode().strip()}"
            logger.error(err_msg)
            console.print(err_msg, style="bold red")
            return

        if stdout:
            lines = stdout.decode().strip().split("\n")
            for line in lines:
                # Verifique se o processo atual corresponde ao nome do processo
                if process_name in line:
                    try:
                        # O PID(Process ID) é a segunda coluna na saída do tasklist
                        pid = int(line.split(",")[1].strip('"'))
                        await asyncio.create_subprocess_exec(
                            "taskkill", "/PID", str(pid), "/F"
                        )
                        log_msg = f"Processo {process_name} (PID {pid}) finalizado."
                        logger.info(log_msg)
                        console.print(
                            f"\n{log_msg}\n",
                            style="bold green",
                        )
                    except Exception as ex:
                        err_msg = f"Erro ao tentar finalizar o processo {process_name} (PID {pid}): {ex}"
                        logger.error(err_msg)
                        console.print(
                            err_msg,
                            style="bold red",
                        )
        else:
            log_msg = f"Nenhum processo chamado {process_name} encontrado para o usuário {current_user}."
            logger.info(
                log_msg,
                None,
            )
            console.print(
                log_msg,
                style="bold yellow",
            )

    except Exception as e:
        err_msg = f"Erro ao tentar matar o processo: {e}"
        logger.error(err_msg)
        console.print(err_msg, style="bold red")


async def find_element_center(image_path, region_to_look, timeout):
    try:
        counter = 0
        confidence_value = 1.00
        grayscale_flag = False

        while counter <= timeout:
            try:
                element_center = pyautogui.locateCenterOnScreen(
                    image_path,
                    region=region_to_look,
                    confidence=confidence_value,
                    grayscale=grayscale_flag,
                )
            except Exception as ex:
                element_center = None
                console.print(
                    f"[{counter+1}] - Elemento não encontrado na posição: {region_to_look}"
                )

            if element_center:
                console.print(
                    f"[{counter+1}] - Elemento encontrado na posição: {region_to_look}\n",
                    style="green",
                )
                return element_center
            else:
                counter += 1

                if confidence_value > 0.81:
                    confidence_value -= 0.01

                if counter >= math.ceil(timeout / 2):
                    grayscale_flag = True

                await worker_sleep(1)

        return None
    except Exception as ex:
        console.print(
            f"{counter} - Buscando elemento na tela: {region_to_look}",
            style="bold yellow",
        )
        return None


def type_text_into_field(text, field, empty_before, chars_to_empty):
    try:
        if empty_before:
            field.type_keys("{BACKSPACE " + chars_to_empty + "}", with_spaces=True)

        field.type_keys(text, with_spaces=True)

        if str(field.texts()[0]) == text:
            return
        else:
            field.type_keys("{BACKSPACE " + chars_to_empty + "}", with_spaces=True)
            field.type_keys(text, with_spaces=True)

    except Exception as ex:
        logger.error("Erro em type_text_into_field: " + str(ex), None)
        console.print(f"Erro em type_text_into_field: {str(ex)}", style="bold red")


async def wait_element_ready_win(element, trys):
    max_trys = 0

    while max_trys < trys:
        try:
            if element.wait("exists", timeout=2):
                await worker_sleep(1)
                if element.wait("exists", timeout=2):
                    await worker_sleep(1)
                    if element.wait("enabled", timeout=2):
                        element.set_focus()
                        await worker_sleep(1)
                        if element.wait("enabled", timeout=1):
                            return True

        except Exception as ex:
            logger.error("wait_element_ready_win -> " + str(ex), None)
            console.print(
                f"Erro em wait_element_ready_win: {str(ex)}", style="bold red"
            )

        max_trys = max_trys + 1

    return False


async def login_emsys(config, app, task):

    from pywinauto.application import Application

    warnings.filterwarnings(
        "ignore",
        category=UserWarning,
        message="32-bit application should be automated using 32-bit Python",
    )

    await worker_sleep(10)
    # await asyncio.sleep(10)
    # Testa se existe alguma mensagem no Emsys
    console.print("Testando se existe alguma mensagem no Emsys...")
    window_message_login_emsys = await find_element_center(
        "assets/emsys/window_message_login_emsys.png", (560, 487, 1121, 746), 15
    )

    # Clica no "Não mostrar novamente" se existir
    console.print("Clicando no 'Não mostrar novamente' se existir...")
    if window_message_login_emsys:
        pyautogui.click(window_message_login_emsys.x, window_message_login_emsys.y)
        pyautogui.click(
            window_message_login_emsys.x + 383, window_message_login_emsys.y + 29
        )
        console.print("Mensagem de login encontrada e fechada.", style="bold green")

    # Ve se o Emsys esta aberto no login
    console.print("Verificando se o Emsys esta aberto no login...")
    image_emsys_login = await find_element_center(
        "assets/emsys/logo_emsys_login.png", (800, 200, 1400, 700), 60
    )

    # if image_emsys_login == None:
    #     image_emsys_login = await find_element_center(
    #     "assets/emsys/logo_emsys_linx_login.png", (800, 200, 1400, 700), 60
    # )

    if image_emsys_login:
        console.print("Aguardando a janela de login ficar pronta...")
        if await wait_element_ready_win(app["Login"]["Edit2"], 80):
            console.print("Procurando o icone disconect_database...")
            disconect_database = await find_element_center(
                "assets/emsys/disconect_database.png", (1123, 452, 1400, 578), 60
            )

            if disconect_database:
                # Realiza login no Emsys
                console.print("Realizando login no Emsys...")
                type_text_into_field(config["user"], app["Login"]["Edit2"], True, "50")
                pyautogui.press("tab")
                type_text_into_field(
                    config["pass"],
                    app["Login"]["Edit1"],
                    True,
                    "50",
                )
                pyautogui.press("enter")

                # Seleciona a filial do emsys
                console.print("Seleciona a filial do emsys...")
                selecao_filial = await find_element_center(
                    "assets/emsys/selecao_filial.png", (480, 590, 820, 740), 15
                )

                console.print(f"Selecao filial via imagem: {selecao_filial}")
                if selecao_filial == None:
                    screenshot_path = take_screenshot()
                    selecao_filial = find_target_position(
                        screenshot_path, "Grupo", 0, -50, attempts=15
                    )
                    console.print(f"Selecao filial localização de texto: {selecao_filial}")
                    if selecao_filial == None:
                        selecao_filial = (700, 639)
                        console.print(f"Selecao filial posição fixa: {selecao_filial}")

                    pyautogui.click(selecao_filial)
                    console.print(f"Escrevendo [{task["configEntrada"]["filialEmpresaOrigem"]}] no campo filial...")
                    pyautogui.write(task["configEntrada"]["filialEmpresaOrigem"])
                    

                else:                    
                    console.print(f"Escrevendo [{task["configEntrada"]["filialEmpresaOrigem"]}] no campo filial...")
                    type_text_into_field(
                        task["configEntrada"]["filialEmpresaOrigem"],
                        app["Seleção de Empresas"]["Edit"],
                        True,
                        "50",
                    )
                pyautogui.press("enter")

                button_logout = await find_element_center(
                    "assets/emsys/button_logout.png", (0, 0, 130, 150), 60
                )

                if button_logout:
                    console.print(
                        "Login realizado com sucesso.", style="bold green"
                    )
                    return {
                        "sucesso": True,
                        "retorno": "Logou com sucesso no emsys!",
                    }
        else:
            log_msg = "Elemento de login não está pronto."
            logger.info(log_msg)
            console.print(log_msg, style="bold red")
            return {"sucesso": False, "retorno": "Falha ao logar no EMSys!"}
    
    else:
        log_msg = "A tela de login nao foi encontrada."
        logger.info(log_msg)
        console.print(log_msg, style="bold red")
        return {"sucesso": False, "retorno": log_msg}


async def api_simplifica(
    urlSimplifica,
    status,
    observacao,
    uuidsimplifica,
    numero_nota,
    valor_nota,
):

    data = {
        "uuid_simplifica": uuidsimplifica,
        "status": status,
        "numero_nota": numero_nota,
        "observacao": observacao,
        "valor_nota": valor_nota,
    }

    try:
        async with aiohttp.ClientSession(
            connector=aiohttp.TCPConnector(verify_ssl=False)
        ) as session:
            async with session.post(f"{urlSimplifica}", data=data) as response:
                data = await response.text()
                log_msg = f"\nSucesso ao enviar {data}\n para o simplifica"
                console.print(
                    log_msg,
                    style="bold green",
                )
                logger.info(log_msg)

    except Exception as e:
        err_msg = f"Erro ao comunicar com endpoint do Simplifica: {e}"
        console.print(f"\n{err_msg}\n", style="bold red")
        logger.info(err_msg)


def add_start_on_boot_to_registry():
    import winreg as reg

    try:
        # Caminho para a chave Run
        registry_path = r"Software\Microsoft\Windows\CurrentVersion\Run"

        # Nome da chave
        key_name = "worker-startup"

        # Caminho para o executável no diretório atual
        directory_value = os.path.join(os.getcwd(), "worker-startup.bat")

        # Acessar a chave de registro
        registry_key = reg.OpenKey(
            reg.HKEY_CURRENT_USER, registry_path, 0, reg.KEY_SET_VALUE
        )

        # Adicionar ou modificar o valor
        reg.SetValueEx(registry_key, key_name, 0, reg.REG_SZ, directory_value)

        # Fechar a chave de registro
        reg.CloseKey(registry_key)

        log_msg = f"Chave {key_name} adicionada ao registro com sucesso com o valor '{directory_value}'!"
        console.print(
            f"\n{log_msg}\n",
            style="bold green",
        )
        logger.info(log_msg)

    except Exception as e:
        err_msg = f"Erro ao adicionar ao registro: {e}"
        console.print(f"\n{err_msg}\n", style="bold red")
        logger.error(err_msg)


def create_worker_bat():
    try:
        # Caminho do diretório atual
        current_dir = os.getcwd()
        nome_arquivo = "worker-startup.bat"

        # Conteúdo do arquivo
        # cd %USERPROFILE%
        bat_content = f"""@echo off
cd {current_dir}   
pipx install worker-automate-hub --force     
start /min "" "worker" "run"
"""

        # Caminho completo para o arquivo
        bat_file_path = os.path.join(current_dir, nome_arquivo)

        # Escrevendo o conteúdo no arquivo
        with open(bat_file_path, "w") as file:
            file.write(bat_content.strip())

        log_msg = f"Arquivo {nome_arquivo} criado com sucesso em {bat_file_path}!"
        console.print(
            f"\n{log_msg}\n",
            style="bold green",
        )
        logger.info(log_msg)

    except Exception as e:
        err_msg = f"Erro ao criar o arquivo {nome_arquivo}: {e}"
        console.print(f"\n{err_msg}\n", style="bold red")
        logger.error(err_msg)


def take_screenshot() -> Path:   
    screenshot_path = Path.cwd() / "temp" / "screenshot.png"      
    screenshot_path.parent.mkdir(parents=True, exist_ok=True)    
    screenshot = pyautogui.screenshot()
    screenshot.save(screenshot_path)
    
    return screenshot_path


def preprocess_image(image_path):
    # Carregar a imagem
    image = cv2.imread(str(image_path))

    # Converter para escala de cinza
    gray = cv2.cvtColor(image, cv2.COLOR_BGR2GRAY)

    # Aplicar threshold binário
    _, binary_image = cv2.threshold(gray, 150, 255, cv2.THRESH_BINARY)

    # Remover ruído com medianBlur
    denoised_image = cv2.medianBlur(binary_image, 3)

    # Aumentar o contraste
    contrast_image = cv2.convertScaleAbs(denoised_image, alpha=1.5, beta=0)

    return contrast_image


def take_target_position(
    screenshot_path: Path, target_text: str, vertical=0, horizontal=0
) -> tuple | None:

    selected_image = Image.open(screenshot_path).convert("L")

    # Configurações do pytesseract
    # custom_config = r'--oem 3 --psm 6'

    # Extrair dados do texto usando pytesseract
    text_data = pytesseract.image_to_data(
        selected_image,
        output_type=pytesseract.Output.DICT,
        lang="por",  # , config=custom_config
    )

    # Identificar a posição do texto desejado
    field_center = None
    for i, text in enumerate(text_data["text"]):
        if len(text) > 0:
            if target_text.lower() in str(text).lower():
                x = text_data["left"][i]
                y = text_data["top"][i]
                w = text_data["width"][i]
                h = text_data["height"][i]
                # Centralizando nas coordenadas do campo
                field_center = (x + w // 2, y + h // 2)
                break

    # Aplicar as modificações de posição
    if field_center:
        field_center = (field_center[0] + horizontal, field_center[1] + vertical)

    return field_center


def find_target_position(
    screenshot_path: Path,
    target_text: str,
    vertical_pos: int = 0,
    horizontal_pos: int = 0,
    attempts: int = 5,
) -> tuple | None:
    attempt = 0
    target_pos = None

    while attempt < attempts:
        target_pos = take_target_position(
            screenshot_path,
            target_text,
            vertical=vertical_pos,
            horizontal=horizontal_pos,
        )
        console.print(f"Tentativa {attempt + 1} - Posição: {target_pos}")
        if target_pos is not None:
            log_msg = f"Posição do campo [{target_text}] encontrada na tentativa [{attempt + 1}], com valor de: {target_pos}"
            console.print(log_msg, style="green")
            logger.info(log_msg)
            return target_pos

        attempt += 1

    # Caso não tenha encontrado após todas as tentativas
    log_msg = f"Não foi possível encontrar o campo [{target_text}] em [{attempts}] tentativas!"
    console.print(log_msg, style="red")

    return None


def select_model_capa():
    screenshot_path = take_screenshot()
    field = find_target_position(screenshot_path, "Documento", 0, 140, 5)
    if field == None:
        return {"sucesso": False, "retorno": f"Não foi possivel encontrar o campo 'Documento'"}
    pyautogui.click(field)
    worker_sleep(1)
    pyautogui.write("Nfe")
    pyautogui.hotkey("enter")
    #Procura o tipo de documento "NFe - NOTA FISCAL ELETRONICA PROPRIA - DANFE SERIE 077"
    while True:
        screenshot_path = take_screenshot()
        field_doc = find_target_position(screenshot_path, "77", 0, 140, 5)
        if field_doc is not None:

            break
        else:
            pyautogui.click(field)
            pyautogui.hotkey("enter")
            pyautogui.hotkey("down")
            pyautogui.hotkey("enter")
            pyautogui.hotkey("tab")
            worker_sleep(2)

    return {"sucesso": True, "retorno": f"Modelo Selecionado"}


async def download_xml(google_drive_folder_id, get_gcp_token, get_gcp_credentials, chave_nota):

    try:
        console.print("Verificando a existência do arquivo no Google Drive...\n")
        chave_nota = f"{chave_nota}.xml"
        gcp_credencial = GetCredsGworkspace(token_dict=get_gcp_token['conConfiguracao'], credentials_dict=get_gcp_credentials['conConfiguracao'])
        creds = gcp_credencial.get_creds_gworkspace()

        if not creds:
            console.print(f'Erro ao obter autenticação para o GCP...\n')
            return {"sucesso": False, "retorno": f"Erro ao obter autenticação para o GCP"}

        # Inicializando o serviço do Google Drive
        drive_service = build("drive", "v3", credentials=creds)

        # Query para procurar o arquivo com o nome da chave da nota
        query = f"'{google_drive_folder_id}' in parents and name contains '{chave_nota}'"
        results = drive_service.files().list(
            q=query,
            pageSize=10,  # Reduzindo o número de resultados
            supportsAllDrives=True,
            includeItemsFromAllDrives=True,
            fields="files(id, name)"
        ).execute()

        # Verificando se o arquivo foi encontrado
        items = results.get('files', [])

        if not items:
            console.print(f'Nenhum arquivo com o nome {chave_nota} foi encontrado...\n')
            return {"sucesso": False, "retorno": f"Nenhum arquivo com o nome {chave_nota} foi encontrado no Google Drive"}

        # Pegando o primeiro arquivo encontrado
        file_id = items[0]['id']
        file_name = items[0]['name']
        console.print(f'Arquivo {file_name} encontrado. Iniciando o download...\n')

        # Definindo o caminho local para salvar o arquivo
        file_path = os.path.join(os.path.expanduser('~'), 'Downloads', file_name)

        # Iniciando o download
        request = drive_service.files().get_media(fileId=file_id)
        fh = io.FileIO(file_path, 'wb')
        downloader = MediaIoBaseDownload(fh, request)

        done = False
        while not done:
            status, done = downloader.next_chunk()
            console.print(f"Download {int(status.progress() * 100)}% concluído.")

        console.print(f"Arquivo {file_name} baixado com sucesso e salvo em {file_path}.\n")
        return {"sucesso": True, "retorno": f"Arquivo {file_name} baixado com sucesso"}

    except Exception as e:
        console.print(f'Erro ao baixar o arquivo do Google Drive, erro: {e}...\n')
        return {"sucesso": False, "retorno": f"Erro ao baixar o arquivo do Google Drive, erro: {e}"}


async def get_xml(xml_file):
    from pywinauto.application import Application

    console.print("Verificando a existencia da tela de importação do XML... \n")
    app = Application().connect(title="Selecione a forma de obter o XML da NF-e")
    main_window = app["Selecione a forma de obter o XML da NF-e"]

    console.print("Selecionando Arquivo XML para seguir com a importação... \n")
    arquivo_xml_button = main_window.child_window(title="Arquivo XML")
    if arquivo_xml_button.is_enabled():
        arquivo_xml_button.click()
        console.print("Arquivo XML selecionado com sucesso... \n")
    else:
        raise {"sucesso": False, "retorno": f"Arquivo XML não foi encontrado."}
    
    console.print("Clicando em OK... \n")
    ok_button = main_window.child_window(title="OK", class_name="TBitBtn")
    if ok_button.is_enabled():
        ok_button.click()  # Clique no botão "OK"
        console.print("Botão OK clicado com sucesso... \n")
    else:
        raise {"sucesso": False, "retorno": f"Botão OK não foi encontrado."}
    
    await worker_sleep(10)

    console.print("Verificando a existencia da tela para importar o xml...\n")
    app = Application().connect(title="Abrir")
    main_window = app["Abrir"]
    
    console.print("Carregando informações do XML a ser importado...\n")
    username = getpass.getuser()
    xml_name = f"{xml_file}.xml"
    path_to_xml = f"C:\\Users\\{username}\\Downloads\\{xml_name}"
    console.print("Inserindo caminho do XML...\n")
    main_window.type_keys('%n')
    await worker_sleep(2)
    pyautogui.write(path_to_xml)
    await worker_sleep(2)
    main_window.type_keys('%a')
    await worker_sleep(2)


async def delete_xml(nfe_key):
    try:
        xml_filename = f"{nfe_key}.xml"
        download_folder = os.path.join(os.path.expanduser("~"), "Downloads")
        file_path = os.path.join(download_folder, xml_filename)
        
        if os.path.exists(file_path):
            os.remove(file_path)
            console.print(f"Arquivo {xml_filename} deletado com sucesso.", style="bold green")
        else:
            console.print(f"Arquivo {xml_filename} não encontrado em {download_folder}.", style="bold yellow")
    except Exception as e:
        console.print(f"Erro ao deletar o arquivo {xml_filename}: {str(e)}", style="bold red")


def config_natureza():
    pyautogui.click(869, 370)
    worker_sleep(1)
    pyautogui.write("16")
    worker_sleep(1)
    pyautogui.press("down", presses=7)
    worker_sleep(1)
    pyautogui.hotkey("enter")
    worker_sleep(1)


def config_almoxarifado(cod_almoxarifado):
    pyautogui.click(841, 390)
    worker_sleep(1)
    pyautogui.write(cod_almoxarifado)
    worker_sleep(1)
    pyautogui.press("tab")
    worker_sleep(1)
    pyautogui.click(1099, 727)


def check_itens_nota():
    pyautogui.click(631, 343)
    worker_sleep(1)
    pyautogui.click(626, 545)
    worker_sleep(1)


def check_pagamento():
    pyautogui.click(623, 374)
    worker_sleep(1)
    pyautogui.click(878, 544)
    worker_sleep(1)
    pyautogui.write("ba")
    worker_sleep(1)
    pyautogui.hotkey("enter")
    worker_sleep(1)


def check_pagamento_transferencia_cd():
    pyautogui.click(623, 374)
    worker_sleep(1)
    pyautogui.click(916, 349)
    worker_sleep(1)
    pyautogui.press("down", presses=19)
    worker_sleep(1)
    pyautogui.hotkey("enter")


def preencher_valor_restante(restante):
    pyautogui.click(1284, 351)
    worker_sleep(1)
    pyautogui.write(restante)
    worker_sleep(1)


def incluir_registro():
    pyautogui.click(594, 297)
    worker_sleep(5)
    pyautogui.click(1225, 635)
    worker_sleep(30)
    pyautogui.click(959, 564)


def finalizar_importacao():
    pyautogui.click(597, 299)
    worker_sleep(1)
    pyautogui.hotkey("enter")


async def importar_notas_outras_empresas(data_emissao, numero_nota, empresa=None):
    #Digita empresa
    data_emissao = data_emissao.replace("/", "")
    if empresa is not None:
        pyautogui.write(empresa)
    else:
        pyautogui.write("171")
    await worker_sleep(1)
    #Digita datas
    pyautogui.click(768, 428)
    await worker_sleep(1)
    pyautogui.write(data_emissao)
    await worker_sleep(1)
    pyautogui.click(859, 430)
    await worker_sleep(1)
    pyautogui.write(data_emissao)
    await worker_sleep(1)
    #Clica Campo 'Num:'"
    pyautogui.click(1014, 428)
    pyautogui.write(numero_nota)
    await worker_sleep(1)
    #Click pesquisar
    pyautogui.click(1190, 428)
    await worker_sleep(20)
    #Click em importar
    pyautogui.click(1207, 684)
    await worker_sleep(20)

# def selecionar_nota_externa(cnpj_fornecedor):
#     screenshot_path = take_screenshot()
#     field = find_target_position(screenshot_path, "de", 30, 0, 15)
#     if field == None:
#         return {"sucesso": False, "retorno": f"Não foi possivel encontrar o campo 'Empresa'"}
#     pyautogui.click(field)
#     pyautogui.write(cnpj_fornecedor)
#     pyautogui.hotkey('tab')


def digitar_datas_emissao(data_emissao):
    screenshot_path = take_screenshot()
    field = find_target_position(screenshot_path, "emissão", 0, 40, 15)
    if field == None:
        return {"sucesso": False, "retorno": f"Não foi possivel encontrar o campo 'Data de emissão inicio'"}
    pyautogui.click(field)
    pyautogui.write(data_emissao)

    field = find_target_position(screenshot_path, "a", 0, 40, 15)
    if field == None:
        return {"sucesso": False, "retorno": f"Não foi possivel encontrar o campo 'Data de emissão fim'"}
    pyautogui.click(field)
    pyautogui.write(data_emissao)


async def import_nfe():
    await worker_sleep(2)
    console.print('Navegando pela Janela de Nota Fiscal de Entrada...\n')
    from pywinauto.application import Application
    
    try:
        importar_nfe = pyautogui.locateOnScreen(ASSETS_PATH + "\\entrada_notas\\ImportarNF-e.png", confidence=0.7)
        if importar_nfe:
            pyautogui.click(importar_nfe)
            await worker_sleep(8)
            return {"sucesso": True, "retorno": f"Clicou Importar nfe"}
        else:
            return {"sucesso": False, "retorno": f"Não foi possivel localizar o campo 'Importar NF-e' "}
    except:
        app = Application().connect(title="Nota Fiscal de Entrada")
        main_window = app["Nota Fiscal de Entrada"]
        tpanel = main_window.child_window(class_name="TPanel")
        tpanel.set_focus()
        await worker_sleep(2)
        
        #Coordernadas relativa a Janela de Notas de entrada e ao filho TPainel
        x_start = 515
        y_start = 1
        x_end = 620
        y_end = 37

        absolute_x = x_start + tpanel.rectangle().left + (x_end - x_start) // 2
        absolute_y = y_start + tpanel.rectangle().top + (y_end - y_start) // 2
        pyautogui.click(absolute_x, absolute_y)
        return {"sucesso": True, "retorno": f"Clicou Importar nfe"}


def digitar_numero_nota(numero_nota):
    screenshot_path = take_screenshot()
    field = find_target_position(screenshot_path, "Núm", 0, 60, 15)
    if field == None:
        return {"sucesso": False, "retorno": f"Não foi possivel encontrar o campo 'Núm Nota'"}
    #pyautogui.click(field)
    pyautogui.write(numero_nota)
    console.log("Escreveu numero da nota", style="bold green")
    field = find_target_position(screenshot_path, "pesquisar", 0 ,0, 15)
    if field == None:
        return {"sucesso": False, "retorno": f"Não foi possivel encontrar o botão 'Pesquisar'"}
    console.log("Clicando em pesquisar", style="bold green")
    pyautogui.click(field)


def select_nfe(nfe_type):
    screenshot_path = take_screenshot()
    field = find_target_position(screenshot_path, nfe_type, 0 ,0, 15)
    if field == None:
        return {"sucesso": False, "retorno": f"Não foi possivel encontrar o campo de 'Notas de Outras Empresas'"}
    pyautogui.click(field)

    return {"sucesso": True, "retorno": f"Selecionou {nfe_type}"}


async def transmitir_nota(task, nota_fiscal, valor_nota):
    pyautogui.click(875, 596)
    logger.info("\nNota Transmitida")
    console.print("\nNota Transmitida", style="bold green")

    await worker_sleep(7)

    #Fechar transmitir nota
    console.print("Fechando a transmissão da nota...\n")

    pyautogui.click(957, 556)

    await worker_sleep(15)
    screenshot_path = take_screenshot()
    transmitir_fechar = find_target_position(screenshot_path, "fechar", attempts=15)
    if transmitir_fechar is not None:
        pyautogui.click(transmitir_fechar)
        log_msg = f'Nota Transmitida com sucesso'
        logger.info(log_msg)
        console.print(log_msg, style="bold green")
    else:
        log_msg = f'Nota não transmitida'
        logger.info(log_msg)
        console.print(log_msg, style='bold red')
        await api_simplifica(task['configEntrada']['urlRetorno'], "ERRO", log_msg, task['configEntrada']['uuidSimplifica'], nota_fiscal, valor_nota)
        return {"sucesso": False, "retorno": log_msg}


    return{'sucesso': True, 'retorno': "Nota transmitida com sucesso"}


async def select_model_pre_venda(text_to_find : str, task):
    #Procura se o campo de select ja possui a opção que você precisa selecionada
    screenshot = take_screenshot()
    field = find_target_position(screenshot, text_to_find, 0, 0, 5)
    if field:
        return True
    else:
        retorno = await find_desired_model(text_to_find, task)
    
    if retorno:
        return retorno


async def extract_value():
    # pyautogui.click(1304, 780)
    logger.info("Extrindo Valor da nota")
    pyautogui.doubleClick(1304, 780, interval=0.3)
    pyautogui.doubleClick(1304, 780, interval=0.3)
    pyautogui.hotkey("ctrl", "c")
    console.log(f"Valor nota copiado: {pyperclip.paste()}", style='bold green')
    valor_nota = pyperclip.paste()
    # valor_nota = re.findall(r'\b\d{1,3}(?:\.\d{3})*,\d{2}\b', valor_nota)
    # print(valor_nota)
    logger.info(valor_nota)
    return valor_nota


async def extract_nf_number():
    logger.info("Extrindo Numero da nota")
    # screenshot_path = take_screenshot()
    # nota_fiscal_gerada = find_target_position(6395, "gerada", attempts=15)
    # pyautogui.click(nota_fiscal_gerada)
    pyautogui.click(965, 515)
    await asyncio.sleep(5)
    pyautogui.hotkey("ctrl", "c")
    nota_fiscal = pyperclip.paste()
    nota_fiscal = re.findall(r'\d+-?\d*', nota_fiscal)
    logger.info(nota_fiscal)
    return nota_fiscal[0]


async def find_desired_model(text_to_find, task):
    #Get index do campo
    from worker_automate_hub.api.client import get_index_modelo_emsys
    index = await get_index_modelo_emsys(task['configEntrada']['filialEmpresaOrigem'], text_to_find)
    console.log(f"Indice do banco: {index['indice']}")
    # #Se não achou clica no campo modelo e sobe ate a primeira opção
    modelo_select_position = (830, 268)
    pyautogui.click(modelo_select_position)
    #Sobe para o primeiro modelo disponivel
    pyautogui.hotkey('enter')
    pyautogui.press('up', presses=20, interval=0.1)
    indice: int = int(index['indice'])
    #Seleciona o modelo com base na query "get_index_modelo_emsys"
    pyautogui.press('down', presses=indice)
    pyautogui.hotkey('tab')
    
    await worker_sleep(3)
    return True
    # screenshot = take_screenshot()
    # field = find_target_position(screenshot, text_to_find, 0, 0, 5)
    # if field:
    #     console.log("Selecionou Modelo da nota corretamente", style='bold green')
    #     return True
    # else:
    #     console.log("Modelo não selecionado corretamente", style='bold red')
    #     return False
    

async def faturar_pre_venda(task):
    #Clica em Faturar
        button_faturar = (1311, 396) #find_target_position(screenshot_path, "Faturar", attempts=15)
        await worker_sleep(2)
        pyautogui.click(button_faturar)
        console.print(f"Clicou em: 'Faturar'",style='bold green')

        await worker_sleep(10)

        #Aviso "Deseja faturar pré-venda?"
        button_yes = (918, 557) #find_target_position(screenshot_path, "yes", attempts=15)
        pyautogui.click(button_yes)

        await worker_sleep(10)

        #Verifica se existe a mensagem de recalcular parcelas
        screenshot_path = take_screenshot()
        message_recalcular = find_target_position(screenshot_path, "Recalcular", attempts=5)
        #Se existir clica em nao
        if message_recalcular is not None:
            button_no = (999, 560) #find_target_position(screenshot_path, "No", attempts=15)
            pyautogui.click(button_no)
            console.log("Cliclou em 'No' na mensagem de recalcular parcelas", style="bold green") 
            logger.info("Cliclou em 'No' na mensagem de recalcular parcelas") 
        else:
            logger.info(f"Mensagem de para recalcular parcelas da pre-venda nao existe")
            console.print(f"Mensagem de para recalcular parcelas da pre-venda nao existe", style="bold yellow")
        
        await worker_sleep(8)

        #Seleciona Modelo
        console.log("Selecionando o modelo...\n", style='bold green')
        retorno = await select_model_pre_venda("077", task)
        if retorno == True:
            console.log('Modelo selecionado com sucesso', style='bold green')
            logger.info('Modelo selecionado com sucesso', style='bold green')
        else:
            await api_simplifica(task['configEntrada']['urlRetorno'], "SUCESSO", retorno['retorno'], task['configEntrada']['uuidSimplifica'], None, None)
            return {"sucesso": False, "retorno": retorno['retorno']}

        #Extrai total da Nota
        console.log("Obtendo o total da Nota...\n", style='bold green')
        valor_nota = await extract_value()
        console.print(f"\nValor NF: '{valor_nota}'", style='bold green')

        #Clicar no botao "OK" com um certo verde
        button_verde = (1180, 822) #find_target_position(screenshot_path, "Ok", attempts=15)
        pyautogui.click(button_verde)
    
        await worker_sleep(5)

        #Clicar no botao "Yes" para "Deseja realmente faturar esta pre-venda?""
        button_yes = (920, 560) #find_target_position(screenshot_path, "Ok", attempts=15)
        pyautogui.click(button_yes)

        return {"sucesso": True, "valor_nota": valor_nota}


async def wait_transmit():
    console.log("Esperando transmissao da nota", style='bold green')
    aguardando_nota = True
    timeout = 35
    timeout_try = 0
    while(aguardando_nota == False or timeout >= timeout_try):
        console.print(f"Tentativa número: {timeout_try}", style="bold yellow")
        logger.info(f"Tentativa número: {timeout_try}")
        await worker_sleep(10)
        screenshot_path = take_screenshot()
        target = find_target_position(screenshot_path, "gerando", 1, 0, 1) 
        if target:
            console.print("Aguardando nota ser transmitida...", style="bold yellow")
            logger.info("Aguardando nota ser transmitida...")
            timeout_try += 1
            continue
        else:
            await worker_sleep(3)
            console.print("Verificando lançamento da nota", style="bold yellow")
            logger.info("Verificando lançamento da nota")
            screenshot_path = take_screenshot()
            target_nota_lancada = find_target_position(screenshot_path, "autorizado", 1, 0, 3)
            if target_nota_lancada:
                console.print("Nota lançada com sucesso.", style="bold green")
                logger.info("Nota lançada com sucesso.")
                #Fechar transmitir nota
                console.print("Fechando a transmissão da nota...\n", style='bold green')
                logger.info("Fechando a transmissão da nota...")
                #Clica em ok "processo finalizado"
                await worker_sleep(2)
                pyautogui.click(957, 556)
                #Clica em  fechar
                await worker_sleep(2)
                pyautogui.click(1200, 667)
                aguardando_nota = False
                break
            else:
                console.print("Nota ainda não lançada", style="bold green")
                logger.log("Nota ainda não lançada")
                timeout_try += 1
    
    return aguardando_nota


async def verify_nf_incuded():
    try:
        nota_incluida = pyautogui.locateOnScreen(ASSETS_PATH + "\\entrada_notas\\nota_fiscal_incluida.png", confidence=0.9)
        if nota_incluida:
            pyautogui.click(959, 562)
            return True
    except Exception as e:
        console.print(f"Error: {e}")
        return False
    


async def rateio_window(nota):
    screenshot_path = take_screenshot()
    
    #Clica em Selecionar todos
    field = find_target_position(screenshot_path, "todos", 0, 0, 15)
    if field == None:
        return {"sucesso": False, "retorno": f"Campo 'Selecionar Todos' não encontrado"}
    pyautogui.click(field)
    await worker_sleep(2)
    
    #Digita "Centro" 1000 + filialEmpresaOrigem
    pyautogui.click(788, 514)
    filial = 1000 + int(nota['filialEmpresaOrigem'])
    pyautogui.write(str(filial))
    pyautogui.hotkey("tab")

    #Marca "Aplicar rateio aos itens selecionados"
    pyautogui.hotkey("space")
    pyautogui.hotkey("tab")

    #Digita % Rateio
    pyautogui.hotkey("ctrl", "a")
    pyautogui.hotkey("del")
    pyautogui.write("100")

    #Clica Incluir registro
    pyautogui.click(1161, 548)
    await asyncio.sleep(20)
    
    #Clica OK
    pyautogui.click(1200, 683)
    await worker_sleep(5)



import subprocess
import pyautogui
import asyncio
import os
from rich.console import Console
from worker_automate_hub.utils.logger import logger
import time
import pygetwindow as gw

console = Console()

def tirar_screenshot(nome_etapa):
    caminho_screenshot = f"{nome_etapa}_{int(time.time())}.png"
    pyautogui.screenshot(caminho_screenshot)
    console.print(f"Screenshot tirada: {caminho_screenshot}")
    return caminho_screenshot

def deletar_screenshots(caminhos_screenshots):
    for caminho in caminhos_screenshots:
        try:
            os.remove(caminho)
            console.print(f"Screenshot deletada: {caminho}")
        except OSError as e:
            console.print(f"Erro ao deletar screenshot {caminho}: {e}")

def fechar_janelas_rdp_sem_ip():
    janelas_rdp = [win for win in gw.getAllTitles() if "Conexão de Área de Trabalho Remota" in win or "Remote Desktop Connection" in win]
    for titulo in janelas_rdp:
        if not any(char.isdigit() for char in titulo):
            console.print(f"Fechando pop-up de conexão sem IP: {titulo}")
            janela = gw.getWindowsWithTitle(titulo)[0]
            janela.close()
            time.sleep(2)

def fechar_janela_existente(ip):
    try:
        janelas_encontradas = gw.getAllTitles()
        for titulo in janelas_encontradas:
            if ip in titulo:
                janela = gw.getWindowsWithTitle(titulo)[0]
                console.print(f"Fechando janela existente: {titulo}")
                janela.activate()
                pyautogui.hotkey('alt', 'f4')
                time.sleep(2)
                break
        else:
            console.print(f"Nenhuma janela encontrada com o IP: {ip}")

        fechar_janelas_rdp_sem_ip()

    except Exception as e:
        console.print(f"Erro ao tentar fechar a janela: {e}", style="bold red")

def restaurar_janelas_rdp():
    janelas_rdp = [win for win in gw.getAllTitles() if "Conexão de Área de Trabalho Remota" in win or "Remote Desktop Connection" in win]
    
    offset_x = 0
    offset_y = 0
    step_x = 30
    step_y = 30

    for titulo in janelas_rdp:
        janela = gw.getWindowsWithTitle(titulo)[0]
        console.print(f"Processando janela: {titulo}")
        if janela.isMinimized:
            janela.restore()
            console.print(f"Janela restaurada: {titulo}")
        else:
            console.print(f"Janela já está aberta: {titulo}")

        janela.moveTo(offset_x, offset_y)

        offset_x += step_x
        offset_y += step_y

        janela.activate()
        time.sleep(2)

def redimensionar_janela_rdp(largura, altura):
    janelas_rdp = [win for win in gw.getAllTitles() if "Conexão de Área de Trabalho Remota" in win or "Remote Desktop Connection" in win]
    
    if janelas_rdp:
        janela_rdp = gw.getWindowsWithTitle(janelas_rdp[0])[0]
        janela_rdp.resizeTo(largura, altura)
        janela_rdp.moveTo(20, 20)
        console.print(f"Janela redimensionada para {largura}x{altura}.")

        janela_rdp.activate()
        janela_rdp.restore()
        time.sleep(1)
    else:
        console.print("Não foi possível encontrar a janela RDP para redimensionar.")

async def conexao_rdp(task):
    caminhos_screenshots = []
    try:
        ip = task["configEntrada"].get("ip", "")
        user = task["configEntrada"].get("user", "")
        password = task["configEntrada"].get("password", "")

        pyautogui.hotkey('win', 'd')
        console.print("1 - Minimizando todas as telas...")
        await asyncio.sleep(2)

        fechar_janela_existente(ip)

        subprocess.Popen('mstsc')
        console.print("2 - Abrindo conexão de trabalho remota...")
        await asyncio.sleep(2)

        redimensionar_janela_rdp(500, 500)

        await asyncio.sleep(2)

        janelas_rdp = [win for win in gw.getAllTitles() if "Conexão de Área de Trabalho Remota" in win or "Remote Desktop Connection" in win]
        if janelas_rdp:
            janela_rdp = gw.getWindowsWithTitle(janelas_rdp[0])[0]
            janela_rdp.activate()
            janela_rdp.restore()
            await asyncio.sleep(1)

        caminhos_screenshots.append(tirar_screenshot("antes_de_inserir_ip"))
        console.print("3 - Inserindo o IP...")
        pyautogui.write(ip)
        await asyncio.sleep(10)
        caminhos_screenshots.append(tirar_screenshot("depois_de_inserir_ip"))
        pyautogui.press('enter')
        await asyncio.sleep(5)
        caminhos_screenshots.append(tirar_screenshot("depois_de_inserir_usuario"))
        await asyncio.sleep(5)

        console.print("5 - Inserindo a Senha...")
        pyautogui.write(password)
        pyautogui.press('enter')
        await asyncio.sleep(10)
        caminhos_screenshots.append(tirar_screenshot("depois_de_inserir_senha"))

        console.print("6 - Apertando left...")
        pyautogui.press('left')
        await asyncio.sleep(2)
        console.print("7 - Apertando Enter...")
        pyautogui.press('enter')
        await asyncio.sleep(20)
        caminhos_screenshots.append(tirar_screenshot("depois_do_certificado"))

        console.print("8 - Minimizando todas as telas no final...")
        pyautogui.hotkey('win', 'd')
        await asyncio.sleep(2)
        caminhos_screenshots.append(tirar_screenshot("depois_de_minimizar_todas"))

        restaurar_janelas_rdp()
        caminhos_screenshots.append(tirar_screenshot("depois_de_restaurar_janelas"))

        deletar_screenshots(caminhos_screenshots)

        return {"sucesso": True, "retorno": "Processo de conexão ao RDP executado com sucesso."}

    except Exception as ex:
        err_msg = f"Erro ao executar conexao_rdp: {ex}"
        logger.error(err_msg)
        console.print(err_msg, style="bold red")
        caminhos_screenshots.append(tirar_screenshot("erro"))
        deletar_screenshots(caminhos_screenshots)
        return {"sucesso": False, "retorno": err_msg}



import asyncio
from rich.console import Console

console = Console()


async def playground(task):
    console.print(f"\nProcesso de teste iniciado: {task}\n", style="green")
    for numero in range(1, 600):
        console.print(f"Etapa [{numero}] de 600", style="green")
        await asyncio.sleep(1)
    console.print(f"Processo de teste finalizado.", style="green")
    return {"sucesso": True, "mensagem": "Processo de teste executado com sucesso"}



import asyncio
import warnings

import pyautogui
from pywinauto.application import Application
from rich.console import Console
from worker_automate_hub.api.client import sync_get_config_by_name

from worker_automate_hub.api.client import get_config_by_name
from worker_automate_hub.config.settings import load_env_config
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.util import (
    check_pagamento,
    check_itens_nota,
    finalizar_importacao,
    import_nfe,
    kill_process,
    login_emsys,
    select_model_capa,
    type_text_into_field,
    get_xml,
    download_xml,
    config_natureza,
    config_almoxarifado,
    delete_xml,
    set_variable,
    get_variable,
    worker_sleep,
)

pyautogui.PAUSE = 0.5
console = Console()


async def entrada_de_notas_9(task):
    """
    Processo que relazia entrada de notas no ERP EMSys(Linx).

    """
    try:
        #Get config from BOF
        config = await get_config_by_name("login_emsys")
        console.print(task)
       
        #Seta config entrada na var nota para melhor entendimento
        nota = task['configEntrada']
        multiplicador_timeout = int(float(task["sistemas"][0]["timeout"]))
        set_variable("timeout_multiplicador", multiplicador_timeout)

        #Abre um novo emsys
        await kill_process("EMSys")
        app = Application(backend='win32').start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        warnings.filterwarnings("ignore", category=UserWarning, message="32-bit application should be automated using 32-bit Python")
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(config['conConfiguracao'], app, task)

        if return_login['sucesso'] == True:
            type_text_into_field('Nota Fiscal de Entrada', app['TFrmMenuPrincipal']['Edit'], True, '50')
            pyautogui.press('enter')
            await worker_sleep(1)
            pyautogui.press('enter')
            console.print(f"\nPesquisa: 'Nota Fiscal de Entrada' realizada com sucesso", style="bold green")
        else:
            logger.info(f"\nError Message: {return_login["retorno"]}")
            console.print(f"\nError Message: {return_login["retorno"]}", style="bold red")
            return return_login
        
        await worker_sleep(10)
        #Procura campo documento
        model  = select_model_capa()
        if model['sucesso'] == True:
            console.log(model['retorno'], style='bold green')
        else:
            return {"sucesso": False, "retorno": f"{model['retorno']}"}

        #Clica em 'Importar-Nfe'
        imported_nfe  = import_nfe()
        if imported_nfe['sucesso'] == True:
            console.log(imported_nfe['retorno'], style='bold green')
        else:
            return {"sucesso": False, "retorno": f"{import_nfe['retorno']}"}

        await worker_sleep(10)

        # Download XML
        get_gcp_token = sync_get_config_by_name("GCP_SERVICE_ACCOUNT")
        get_gcp_credentials = sync_get_config_by_name("GCP_CREDENTIALS")
        env_config, _ = load_env_config()

        download_xml(env_config["XML_DEFAULT_FOLDER"], get_gcp_token, get_gcp_credentials, nota["nfe"])

        # Permanece 'XML'
        #Clica em  'OK' para selecionar
        pyautogui.click(970, 666)
        await worker_sleep(3)

        # Click Downloads
        get_xml(nota["nfe"])

        # Deleta o xml
        delete_xml(nota["nfe"])

        # Configura a natureza
        config_natureza()

        # Configura o almoxarifado
        config_almoxarifado(nota["observacoes"].split(" ")[0])

        # Verifica os itens na nota
        check_itens_nota()

        # Verifica se as configs de pagamento estao ok
        check_pagamento()

        # Finaliza a importacao
        finalizar_importacao()

    except Exception as ex:
        observacao = f"Erro Processo Entrada de Notas: {str(ex)}"
        logger.error(observacao)
        console.print(observacao, style="bold red")
        return {"sucesso": False, "retorno": observacao}



import asyncio
from pathlib import Path
import re
import io
import shutil
import uuid
import os
import time
import timedelta
from datetime import datetime
import zipfile
import smtplib
import mimetypes
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.base import MIMEBase
from email import encoders


import pandas as pd
import openpyxl
from playwright.async_api import async_playwright

import pyautogui
from rich.console import Console
import psutil

from worker_automate_hub.api.client import get_config_by_name
from worker_automate_hub.utils.logger import logger

from worker_automate_hub.utils.get_creds_gworkspace import GetCredsGworkspace
from googleapiclient.http import MediaIoBaseDownload
from googleapiclient.discovery import build
from worker_automate_hub.api.client import sync_get_config_by_name


console = Console()


def fechar_aplicativo(nome_processo):
    encontrado = False
    for proc in psutil.process_iter(['name']):
        if proc.info['name'] == nome_processo:
            console.print(f"Fechando o processo {nome_processo}...", style="bold red")
            proc.terminate()
            proc.wait()
            encontrado = True
            break
    
    if not encontrado:
        console.print(f"O processo {nome_processo} não está em execução.", style="bold yellow")



def find_and_click(image_path, description, confidence=0.8, write_text=None, wait_time=2):
    element = pyautogui.locateCenterOnScreen(image_path, confidence=confidence)
    if element:
        pyautogui.click(element)
        console.print(f"[green]'{description}' encontrado e clicado.[/green]")
        if write_text:
            pyautogui.write(write_text, interval=0.05)
            console.print(f"[green]Texto '{write_text}' inserido em '{description}'.[/green]")
        time.sleep(wait_time)
    else:
        raise Exception(f"[red]Elemento '{description}' não encontrado![/red]")


def report_to_user(filename,report,path_directory):
    fullpath = os.path.join(path_directory, filename)

    wb = openpyxl.Workbook()
    ws = wb.active

    headers = ["Número do processo", "Assunto", "Tipo de Comunicação", "Data Final Ciência", "Forma Ciência", "Detalhamento", "Inteiro Teor"]
    for col_num, header in enumerate(headers, start=1):
        ws.cell(row=1, column=col_num).value = header

    row_num = 2
    for page_data in report:
        for data in page_data:
            col_num = 1
            for key in headers:
                ws.cell(row=row_num, column=col_num).value = data.get(key, "")
                col_num += 1
            row_num += 1

    wb.save(fullpath)


def compactar_diretorio(diretorio, nome_arquivo_zip):
    arquivos = [os.path.join(diretorio, arquivo) for arquivo in os.listdir(diretorio) if os.path.isfile(os.path.join(diretorio, arquivo))]

    with zipfile.ZipFile(nome_arquivo_zip, 'w') as zipf:
        for arquivo in arquivos:
            zipf.write(arquivo, os.path.basename(arquivo))


def send_email(smtp_server, smtp_port, smtp_user, smtp_password, message_text, subject, to, diretorio):

    nome_arquivo_zip = 'Consulta DJE.zip'
    full_path = os.path.join(diretorio, nome_arquivo_zip)
    compactar_diretorio(diretorio, full_path)

    message = create_message_with_attachment(smtp_user, to, subject, message_text, full_path)
    send_message(smtp_server, smtp_port, smtp_user, smtp_password, smtp_user, to, message)


def create_message_with_attachment(sender, to, subject, message_text, file):
    message = MIMEMultipart()
    message['to'] = to
    message['from'] = sender
    message['subject'] = subject

    msg = MIMEText(message_text)
    message.attach(msg)

    if file:
        content_type, encoding = mimetypes.guess_type(file)

        if content_type is None or encoding is not None:
            content_type = 'application/octet-stream'

        main_type, sub_type = content_type.split('/', 1)
        with open(file, 'rb') as f:
            attachment = MIMEBase(main_type, sub_type)
            attachment.set_payload(f.read())
            encoders.encode_base64(attachment)
            attachment.add_header('Content-Disposition', 'attachment', filename=os.path.basename(file))
            attachment.add_header('Content-Transfer-Encoding', 'base64')
            message.attach(attachment)

    # raw_message = base64.urlsafe_b64encode(message.as_bytes())
    # raw_message = raw_message.decode()
    # return {'raw': raw_message}
    return message.as_string()


def send_message(smtp_server, smtp_port, smtp_user, smtp_password, sender, recipient, message):
    try:
        with smtplib.SMTP(smtp_server, smtp_port) as server:
            server.starttls()
            server.login(smtp_user, smtp_password)
            server.sendmail(sender, recipient, message)
            logger.info('Mensagem enviada com sucesso')
    except Exception as error:
        logger.error('Ocorreu um erro ao enviar a mensagem: %s' % error)


async def coleta_dje_start_update(task):
    from pywinauto.application import Application
    """
    Processo que realiza a consulta de processo no DJE.

    """
    try:
        # Obtém a resolução da tela
        screen_width, screen_height = pyautogui.size()
        console.print(f"Resolução da tela: Width: {screen_width} - Height{screen_height}...\n")
        console.print(f"Task:{task}")

        console.print("Realizando as validações inicias para execução do processo\n")

        console.print("Criando o diretório temporario ...\n")
        cd = Path.cwd()
        temp_dir = f"{cd}\\temp_certificates\\{str(uuid.uuid4())}"
        if not os.path.exists(temp_dir):
            os.makedirs(temp_dir)

        console.print("Obtendo configuração para execução do processo ...\n")
        try:
            config = await get_config_by_name("DJE")
            emails_to = config['conConfiguracao']['emails']
            certificado_path = config['conConfiguracao']['CertificadoPath']
            planilha_id = config['conConfiguracao']['SpreadSheets']

            console.print("Obtendo configuração de email para execução do processo ...\n")
            smtp_config = await get_config_by_name("SMTP")

            smtp_server = smtp_config['conConfiguracao']['server']
            smtp_user = smtp_config['conConfiguracao']['user']
            smtp_pass = smtp_config['conConfiguracao']['password']
            smtp_port =  smtp_config['conConfiguracao']['port']

            get_gcp_token = sync_get_config_by_name("GCP_SERVICE_ACCOUNT")
            get_gcp_credentials = sync_get_config_by_name("GCP_CREDENTIALS")
        except Exception as e:
            console.print(f'Erro ao obter as configurações para execução do processo, erro: {e}...\n')
            return {"sucesso": False, "retorno": f"Erro ao obter as configurações para execução do processo, erro: {e}"}

        try:
            console.print("Realizando a leitura da planilha com os certificados...\n")
            gcp_credencial = GetCredsGworkspace(token_dict=get_gcp_token['conConfiguracao'], credentials_dict=get_gcp_credentials['conConfiguracao'])
            creds = gcp_credencial.get_creds_gworkspace()

            if not creds:
                console.print(f'Erro ao obter autenticação para o GCP...\n')
                return {"sucesso": False, "retorno": f"Erro ao obter autenticação para o GCP"}

            spreadsheet_id = planilha_id
            sheet_name = 'ECACFederal'

            service = build("sheets", "v4", credentials=creds, static_discovery=False)
            sheet = service.spreadsheets()
            result = (
                sheet.values()
                .get(spreadsheetId=spreadsheet_id, range=f"{sheet_name}")
                .execute()
            )
            values = result.get("values", [])
        except Exception as e:
            console.print(f'Não foi possivel obter a planilha no Google Drive para execução do processo, erro: {e}...\n')
            return {"sucesso": False, "retorno": f"Não foi possivel obter a planilha no Google Drive para execução do processo, erro: {e}"}


        console.print("Verificando a existência do diretório com locators...\n")
        if os.path.exists("assets/dje"):
            console.print('Diretório existe..\n')
        else:
            console.print('Diretório não existe..\n')
            return {"sucesso": False, "retorno": f"Não foi possivel encontrar o diretório com os locators para continuidade do processo, diretório: 'assets/dje'"}
        

        console.print('Verificando a existência dos arquivos de certificados no Google Drive...\n')
        drive_service = build("drive", "v3", credentials=creds)
        query = f"'{certificado_path}' in parents"
        results = drive_service.files().list(
            q=query,
            pageSize=1000,
            supportsAllDrives=True,  # Habilita suporte para Shared Drives
            includeItemsFromAllDrives=True,  # Inclui itens de todos os drives
            fields="files(id, name)"
        ).execute()

        items = results.get('files', [])

        if not items:
            console.print(f'Nenhum certificado encontrado...\n')
            return {"sucesso": False, "retorno": f"Nenhum certificado encontrado no diretório do Google Drive"}


        certificate_files = pd.DataFrame(values[1:], columns=values[0])
        report = []
        feedback_cnpj = []
        count_error = 0

        for index, row in certificate_files.iterrows():
            try:
                empresa = row['Empresa']
                certificado = row['Certificado']
                senha = row['Senha']
                console.print(f'Certificado a ser executado: {certificado} para a empresa {empresa}...\n')

                nome_aplicativo = "javaw.exe"
                console.print("\nPJeOffice PRO encerrando...", style="bold green")
                fechar_aplicativo(nome_aplicativo)

                # Abre o PJeOffice
                console.print("\nPJeOffice PRO iniciando...", style="bold green")
                try:
                    app = Application().start("C:\\Program Files\\PJeOffice Pro\\pjeoffice-pro.exe")
                except Exception as e:
                    log_msg = f"Erro Processo do DJE, não foi possivel iniciar a aplicação: {e}"
                    console.print(log_msg, style="bold red")
                    return {"sucesso": False, "retorno": log_msg}
                
                console.print(f'Procurando certificado: {certificado} para a empresa {empresa}...\n')
                drive_service = build("drive", "v3", credentials=creds)
                query = f"'{certificado_path}' in parents and name='{certificado}'"
                results = drive_service.files().list(
                    q=query,
                    pageSize=1000,
                    supportsAllDrives=True,  # Habilita suporte para Shared Drives
                    includeItemsFromAllDrives=True,  # Inclui itens de todos os drives
                    fields="files(id, name)"
                ).execute()

                items = results.get('files', [])

                if items:
                    console.print(f'Certificado: {certificado} para a empresa {empresa} encontrado...\n')
                    file_id = items[0]['id']
                    console.print(f'Certificado {certificado} encontrado. Iniciando download...\n')

                    file_path = os.path.join(temp_dir, certificado)

                    request = drive_service.files().get_media(fileId=file_id)

                    fh = io.FileIO(file_path, 'wb')
                    downloader = MediaIoBaseDownload(fh, request)
    
                    done = False
                    while done is False:
                        status, done = downloader.next_chunk()
                        console.print(f"Download {int(status.progress() * 100)}%.")
                    
                    fh.close()

                    console.print(f"Certificado {certificado} baixado com sucesso.\n")

                    locator_path = "assets/dje"
                    autorizacao_site = f'{locator_path}/AutorizacaoSite_Pje.png'
                    sim_destavez = f'{locator_path}/Sim_destavez.png'
                    informar_senha_existe = f'{locator_path}/InformeSenha.png'
                    cancelar = f'{locator_path}/Cancelar.png'
                    certificados_disponiveis = f'{locator_path}/CertificadosDisponiveis.png'
                    engrenagem_certificado = f'{locator_path}/Engrenagem_Configuracao.png'
                    tipo_certificado = f'{locator_path}/TipoCertificado.png'
                    remover_certificado = f'{locator_path}/RemoverCertificado.png'
                    localizar = f'{locator_path}/SelecionarCertificado.png'
                    cert_fullpath = f'{locator_path}/DiretorioCertificado.png'
                    confirmar_certificado = f'{locator_path}/Abrir.png'
                    ok_confirmar_certificado = f'{locator_path}/OK_Importar.png'
                    senha_certificado = f'{locator_path}/DigitarSenha.png'


                    async with async_playwright() as p:
                        browser = await p.firefox.launch(headless=False)
                        context = await browser.new_context()
                        page = await context.new_page()
                        await page.goto("https://sso.cloud.pje.jus.br/auth/realms/pje/protocol/openid-connect/auth?client_id=domicilio-eletronico-frontend&redirect_uri=https%3A%2F%2Fdomicilio-eletronico.pdpj.jus.br%2Fcomunicacoes%3Btipo%3DComunicacao&state=35cb2d31-0ded-44f2-88e0-d4a02c3b445b&response_mode=fragment&response_type=code&scope=openid&nonce=b38f5525-8e57-4aea-8fbd-e7f3cfebc9ef")
                        await page.get_by_text("Seu certificado digital").click()
                        await asyncio.sleep(3)

                        validador_site = pyautogui.locateOnScreen(autorizacao_site, confidence=0.8)
    
                        if validador_site:
                            find_and_click(sim_destavez, "Sim desta vez")
                        
                        try:
                            while True:
                                solicitacao_senha = pyautogui.locateOnScreen(informar_senha_existe, confidence=0.8)
                                if solicitacao_senha:
                                    pyautogui.click(pyautogui.locateCenterOnScreen(cancelar, confidence=0.8))
                                    await asyncio.sleep(1)
                                else:
                                    break
                        except Exception as e:
                            pass
                        
                        time.sleep(2)
                        
                        certificados_pje = pyautogui.locateOnScreen(certificados_disponiveis, confidence=0.8)
                        if certificados_pje:
                            find_and_click(engrenagem_certificado, "Engrenagem, menu de configuração")
                            await asyncio.sleep(3)
                        
                        find_and_click(tipo_certificado, "Tipo certificado...")
                        await asyncio.sleep(2)
                        
                        try:
                            while True:
                                remover_certificado_exist = pyautogui.locateOnScreen(remover_certificado, confidence=0.7)
                                if remover_certificado_exist:
                                    find_and_click(remover_certificado, "Remover Certificados")
                                    console.print('Certificado ja importado, removendo...\n')
                                    await asyncio.sleep(2)
                                else:
                                    console.print('Certificado ja importado, removidos...\n')
                                    break
                        except Exception as e:
                            pass

                        find_and_click(localizar, "Importar certificado..")
                        find_and_click(cert_fullpath, "Inserir Caminho do Certificado", write_text=file_path)
                        find_and_click(confirmar_certificado, "Confirmar..")

                        remover_certificado_exist = pyautogui.locateOnScreen(remover_certificado, confidence=0.7)
                        if remover_certificado_exist:
                            find_and_click(ok_confirmar_certificado, "OK Confirmar..")
                        
                        await asyncio.sleep(3)
                        
                        if informar_senha_existe:
                            find_and_click(senha_certificado, "Inserir Senha do certificado", write_text=senha)
                            await asyncio.sleep(1)
                            find_and_click(ok_confirmar_certificado, "Confirmar seleção")
                        
                        await page.wait_for_load_state("networkidle")
                        await page.wait_for_selector("text=Comunicação Processual Acompanhe as citações, intimações e ofícios direcionados", timeout=15000)
                        await page.get_by_text("Comunicação Processual Acompanhe as citações, intimações e ofícios direcionados").click()

                        await page.wait_for_load_state("networkidle")
                        await page.wait_for_selector("button.buscar", timeout=15000)
                        await asyncio.sleep(3)
                        page.locator("button.buscar").click()

                        await asyncio.sleep(15)
                        await page.wait_for_load_state("networkidle")

                        # Captura todas as linhas da tabela
                        linhas = await page.query_selector_all('table tbody tr')

                        # Extrai informações de cada linha
                        for linha in linhas:
                            colunas = linha.query_selector_all('td')
                            dados_linha = [coluna.inner_text().strip() for coluna in colunas]

                            if dados_linha and len(dados_linha) > 1:
                                nova_linha = [certificado, empresa] + dados_linha[1:]
                                report.append(nova_linha)
                        
                        await page.get_by_label("Usuário Autenticado").click()
                        await page.locator("a").filter(has_text="Sair").click()

                        await asyncio.sleep(2)

                        await context.close()
                        await browser.close()

                        result = {"error": False, "message": f"Proceso completado com sucesso para o certificado {certificado}"}
                        feedback_cnpj.append(f"Processo finalizado com sucesso para o certificado: {certificado}")

                else:
                    count_error += 1
                    console.print(
                    f"Erro para o {certificado} - Certificado não encontrado no Google Drive"
                    )
                    result = {"error": True, "message": 'Certificado não encontrado no Google Drive', "certificado": certificado}
                    feedback_cnpj.append(f"Certificado não encontrado no Google Drive: {certificado}")


            except Exception as e:
                count_error += 1
                console.print(
                    f"Erro para {certificado}: {str(e)}, Mensagem: {e}"
                )
                result = {"error": True, "error_message": str(e), "certificate": certificado}
                feedback_cnpj.append(f" Não foi possivel concluir o processo para o centificado: {certificado}")


        if count_error == len(certificate_files):
            log_msg = f"Erro Processo do DJE, não obtivemos exito em nenhum certificado executado, erro: {feedback_cnpj}"
            logger.error(log_msg)
            console.print(log_msg, style="bold red")
            return {"sucesso": False, "retorno": log_msg}
        else:        
            report_to_user('consultaDJE.xlsx', report, os.path.join(temp_dir))
            feedback_cnpj_str = '\n'.join(feedback_cnpj)
            corpo_email = f''' Caros(as), \nEspero que este e-mail o encontre bem ! \n\n Abaixo, segue um resumo de facil acesso as consultas do DJE.
            \n Resumo:
            1 - Para consulta temos acesso a {len(certificate_files)} certificados e o RPA tem a responsabilidade de acessar diariamente o sistema DJE para consulta.\n
            CNPJs buscados e seus respectivos status: \n
            {feedback_cnpj_str}

            Feedback de execução em anexo:
            '''
            corpo_email += f'\nEstamos disponivel para quaisquer duvidas ou sugestões.\n'

            try:
                send_email(smtp_server,smtp_port,smtp_user,smtp_pass,corpo_email,'RPA - DJE',emails_to,temp_dir)
                log_msg = f"Processo concluido e e-mail disparado para area de negócio"
                console.print(log_msg)
                return {"sucesso": True, "retorno": log_msg}
            except Exception as e:
                log_msg = f"Processo concluido porém não foi possivel enviar o e-mail para area de negóciol: {e}"
                logger.error(log_msg)
                console.print(log_msg, style="bold red")
                return {"sucesso": False, "retorno": log_msg}
            
    except Exception as e:
        log_msg = f"Erro Processo do DJE: {e}"
        logger.error(log_msg)
        console.print(log_msg, style="bold red")
        return {"sucesso": False, "retorno": log_msg}

    finally:
        if os.path.exists(temp_dir):
            shutil.rmtree(temp_dir)



from worker_automate_hub.tasks.jobs.ecac_estadual_go import ecac_estadual_go
from worker_automate_hub.tasks.jobs.ecac_estadual_mt import ecac_estadual_mt
from worker_automate_hub.tasks.jobs.ecac_estadual_sp import ecac_estadual_sp
from worker_automate_hub.tasks.jobs.ecac_estadual_sc import ecac_estadual_sc


async def ecac_estadual_main(task):
    try:
        task_process = task['configEntrada']['Estado']
    except Exception as e:
        return {"sucesso": False, "retorno": f"Não foi possivel iniciar o processo, pois o Estado não foi provisionado"}

    if task_process == 'GO':
        result = await ecac_estadual_go(task)
        return result
    elif task_process == 'MT':
        result = await ecac_estadual_mt(task)
        return result
    elif task_process == 'SP':
        result = await ecac_estadual_sp(task)
        return result
    elif task_process == 'SC':
        result = await ecac_estadual_sc(task)
        return result
    else:
        return {"sucesso": False, "retorno": f"Estado não mapeado ou desenvolvimento não concluido, {task_process}"}


import asyncio
from pathlib import Path
import io
import shutil
import os
import time
import uuid
import smtplib
import mimetypes
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.base import MIMEBase
from email import encoders

from playwright.async_api import async_playwright

import pyautogui
from rich.console import Console

from worker_automate_hub.api.client import sync_get_config_by_name
from worker_automate_hub.api.client import get_config_by_name
from worker_automate_hub.utils.logger import logger

from worker_automate_hub.utils.get_creds_gworkspace import GetCredsGworkspace
from googleapiclient.http import MediaIoBaseDownload
from googleapiclient.discovery import build



console = Console()


def find_and_click(image_path, description, confidence=0.8, write_text=None, wait_time=2):
    element = pyautogui.locateCenterOnScreen(image_path, confidence=confidence)
    if element:
        pyautogui.click(element)
        console.print(f"[green]'{description}' encontrado e clicado.[/green]")
        if write_text:
            pyautogui.write(write_text, interval=0.05)
            console.print(f"[green]Texto '{write_text}' inserido em '{description}'.[/green]")
        time.sleep(wait_time)
    else:
        raise Exception(f"[red]Elemento '{description}' não encontrado![/red]")


def send_email(smtp_server, smtp_port, smtp_user, smtp_password, message_text, subject, to):
    message = create_message_with_attachment(smtp_user, to, subject, message_text)
    send_message(smtp_server, smtp_port, smtp_user, smtp_password, smtp_user, to, message)


def create_message_with_attachment(sender, to, subject, message_text, file):
    message = MIMEMultipart()
    message['to'] = to
    message['from'] = sender
    message['subject'] = subject

    msg = MIMEText(message_text)
    message.attach(msg)

    if file:
        content_type, encoding = mimetypes.guess_type(file)

        if content_type is None or encoding is not None:
            content_type = 'application/octet-stream'

        main_type, sub_type = content_type.split('/', 1)
        with open(file, 'rb') as f:
            attachment = MIMEBase(main_type, sub_type)
            attachment.set_payload(f.read())
            encoders.encode_base64(attachment)
            attachment.add_header('Content-Disposition', 'attachment', filename=os.path.basename(file))
            attachment.add_header('Content-Transfer-Encoding', 'base64')
            message.attach(attachment)

    return message.as_string()


def send_message(smtp_server, smtp_port, smtp_user, smtp_password, sender, recipient, message):
    try:
        with smtplib.SMTP(smtp_server, smtp_port) as server:
            server.starttls()
            server.login(smtp_user, smtp_password)
            server.sendmail(sender, recipient, message)
            logger.info('Mensagem enviada com sucesso')
    except Exception as error:
        logger.error('Ocorreu um erro ao enviar a mensagem: %s' % error)


async def ecac_estadual_go(task):
    """
    Processo que realiza a consulta de caixa postal do ECAC Estadual de Goais para o certificado SIM DISTRIBUIDORA.

    """
    try:
        # Obtém a resolução da tela
        screen_width, screen_height = pyautogui.size()
        console.print(f"Resolução da tela: Width: {screen_width} - Height{screen_height}...\n")
        console.print(f"Task:{task}")

        console.print("Realizando as validações inicias para execução do processo\n")
        console.print("Criando o diretório temporario ...\n")

        cd = Path.cwd()
        temp_dir = f"{cd}\\temp_certificates\\{str(uuid.uuid4())}"
        if not os.path.exists(temp_dir):
            os.makedirs(temp_dir)

        console.print("Obtendo configuração para execução do processo ...\n")
        try:
            config = await get_config_by_name("Ecac_Estadual_GO")
            emails_to = config['conConfiguracao']['emails']
            certificado_path = config['conConfiguracao']['CertificadoPath']
            certificado_file = config['conConfiguracao']['CertificadoFile']
            certificado_pwd = config['conConfiguracao']['Pwd']

            console.print("Obtendo configuração de email para execução do processo ...\n")
            smtp_config = await get_config_by_name("SMTP")

            smtp_server = smtp_config['conConfiguracao']['server']
            smtp_user = smtp_config['conConfiguracao']['user']
            smtp_pass = smtp_config['conConfiguracao']['password']
            smtp_port =  smtp_config['conConfiguracao']['port']

            get_gcp_token = sync_get_config_by_name("GCP_SERVICE_ACCOUNT")
            get_gcp_credentials = sync_get_config_by_name("GCP_CREDENTIALS")
        except Exception as e:
            console.print(f'Erro ao obter as configurações para execução do processo, erro: {e}...\n')
            return {"sucesso": False, "retorno": f"Erro ao obter as configurações para execução do processo, erro: {e}"}

        console.print("Verificando a existência do diretório com locators...\n")
        if os.path.exists("assets/ecac_federal"):
            console.print('Diretório existe..\n')
        else:
            console.print('Diretório não existe..\n')
            return {"sucesso": False, "retorno": f"Não foi possivel encontrar o diretório com os locators para continuidade do processo, diretório: 'assets/ecac_federal'"}

        try:
            gcp_credencial = GetCredsGworkspace(token_dict=get_gcp_token['conConfiguracao'], credentials_dict=get_gcp_credentials['conConfiguracao'])
            creds = gcp_credencial.get_creds_gworkspace()

            console.print(f'Procurando certificado: {certificado_file} ...\n')
            drive_service = build("drive", "v3", credentials=creds)
            query = f"'{certificado_path}' in parents and name='{certificado_file}'"
            results = drive_service.files().list(
                q=query,
                pageSize=1000,
                supportsAllDrives=True,  # Habilita suporte para Shared Drives
                includeItemsFromAllDrives=True,  # Inclui itens de todos os drives
                fields="files(id, name)"
            ).execute()

            items = results.get('files', [])

            if items:
                console.print(f'Certificado: {certificado_file} encontrado...\n')
                file_id = items[0]['id']
                console.print(f'Iniciando download...\n')

                file_path = os.path.join(temp_dir, certificado_file)

                request = drive_service.files().get_media(fileId=file_id)

                fh = io.FileIO(file_path, 'wb')
                downloader = MediaIoBaseDownload(fh, request)

                done = False
                while done is False:
                    status, done = downloader.next_chunk()
                    console.print(f"Download {int(status.progress() * 100)}%.")

                console.print(f"Certificado {certificado_file} baixado com sucesso.\n")

                fh.close()
            
            else:
                console.print(f'Certificado não encontrado para realização do download, {certificado_file}...\n')
                return {"sucesso": False, "retorno": f"Certificado não encontrado para realização do download, {certificado_file}"}
        except Exception as e:
            console.print(f'Erro ao realizar download do certificado: {e}...\n')
            return {"sucesso": False, "retorno": f"Erro ao realizar download do certificado: {e}"}
                
        
        async with async_playwright() as p:
            browser = await p.firefox.launch(headless=False)
            
            console.print("Iniciando o browser")
            context = await browser.new_context(accept_downloads=True)
            page = await context.new_page()

            await asyncio.sleep(3)

            await page.goto('about:preferences#privacy')

            locator_path = "assets/ecac_federal"

            element_find_settings_image = f'{locator_path}/FindInSettings.PNG' 
            element_view_certificates = f'{locator_path}/ViewCertificates.PNG'
            element_your_certificates = f'{locator_path}/YourCertificates.PNG'
            element_import_certificates = f'{locator_path}/ImportCertificate.PNG'
            element_insert_path_certificates = f'{locator_path}/InsertPathCertificate.PNG'
            element_open_certificates = f'{locator_path}/OpenCertificate.PNG'
            element_pwd_certificates = f'{locator_path}/EnterPasswordCertificate.PNG'
            element_sign_in_certificates = f'{locator_path}/SignCertificate.PNG'
            element_confirm_certificates = f'{locator_path}/Confirm.PNG'
            element_sign_in_certificates = f'{locator_path}/SignCertificate.PNG'
            element_popup_certificate = f'{locator_path}/ImportCertificate_popup.PNG'

            console.print("Importando o certificado no browser (playwright) ")
            # Configurações
            find_and_click(element_find_settings_image, "Configurações")
            pyautogui.write('Cert', interval=0.05)
            await asyncio.sleep(1)

            # Certificados
            find_and_click(element_view_certificates, "Certificados")
            await asyncio.sleep(1)

            # Meus Certificados
            find_and_click(element_your_certificates, "Meus Certificados")
            await asyncio.sleep(1)

            # Importar Certificado
            find_and_click(element_import_certificates, "Importar Certificado")
            await asyncio.sleep(2)

            # Inserir Caminho do Certificado e escrever o caminho do arquivo
            find_and_click(element_insert_path_certificates, "Inserir Caminho do Certificado", write_text=file_path)
            await asyncio.sleep(2)

            # Abrir Certificado
            find_and_click(element_open_certificates, "Abrir Certificado")
            await asyncio.sleep(2)

            # Inserir Senha do Certificado
            find_and_click(element_pwd_certificates, "Inserir Senha do Certificado", write_text=certificado_pwd)
            await asyncio.sleep(2)

            # Assinar Certificado
            find_and_click(element_sign_in_certificates, "Assinar Certificado")
            await asyncio.sleep(2)

            # Confirmar Importação
            find_and_click(element_confirm_certificates, "Confirmar Importação")
            await asyncio.sleep(2)

            element_popup_certificate = pyautogui.locateCenterOnScreen(element_popup_certificate, confidence=0.8)

            if not element_popup_certificate:
                await asyncio.sleep(3)

                await page.goto("https://dte.sefaz.go.gov.br/dte/web/https/painelControle.jsf")
                await asyncio.sleep(3)
                await page.keyboard.press("Enter")

                parte1 = await page.locator("/html/body/div[3]/div/div/div/div/div/div[2]/div[1]/div[2]/form/div[2]/div/div[1]/div[1]/span[1]").text_content()
                parte2 = await page.locator("/html/body/div[3]/div/div/div/div/div/div[2]/div[1]/div[2]/form/div[2]/div/div[1]/div[1]/b[1]/span").text_content()
                parte3 = await page.locator("/html/body/div[3]/div/div/div/div/div/div[2]/div[1]/div[2]/form/div[2]/div/div[1]/div[1]/span[2]").text_content()
                parte4 = await page.locator("/html/body/div[3]/div/div/div/div/div/div[2]/div[1]/div[2]/form/div[2]/div/div[1]/div[1]/b[2]/span").text_content()
                parte5 = await page.locator("/html/body/div[3]/div/div/div/div/div/div[2]/div[1]/div[2]/form/div[2]/div/div[1]/div[1]/span[4]").text_content()

                frase_completa = f"{parte1} {parte2} {parte3} {parte4} {parte5}"

                corpo_email = f''' Caros(as), \nEspero que este e-mail o encontre bem! \n\n Abaixo, segue um resumo baseado na consulta à caixa postal referente ao estado de GO.
                \n Resultado da consulta: \n {frase_completa} \n '''

                try:
                    await context.close()
                    await browser.close()
                except Exception as e:
                    console.print('Erro ao encerrar o browser... \n', e)

                try:
                    send_email(smtp_server, smtp_port, smtp_user, smtp_pass, corpo_email, "Consulta ECAC_Estadual - GO", emails_to)
                    log_msg = f"Processo concluído e e-mail disparado para área de negócio"
                    console.print(log_msg)
                    return {"sucesso": True, "retorno": log_msg}
                except Exception as e:
                    log_msg = f"Processo concluído com sucesso, porém houve falha no envio do e-mail para área de negócio"
                    console.print(log_msg)
                    return {"sucesso": False, "retorno": log_msg}

                         
    except Exception as e:
        if os.path.exists(temp_dir):
            shutil.rmtree(temp_dir)
        log_msg = f"Erro Processo do Ecac Estadual GO: {e}"
        logger.error(log_msg)
        console.print(log_msg, style="bold red")
        return {"sucesso": False, "retorno": log_msg}